<?php

require_once dirname(__FILE__) . '/classes/LVPPluginHelper.class.php';
require_once dirname(__FILE__) . '/classes/LVPPluginView.class.php';
require_once dirname(__FILE__) . '/classes/LVPPluginViewOverview.class.php';
require_once dirname(__FILE__) . '/classes/LVPPluginViewCalendar.class.php';
require_once dirname(__FILE__) . '/classes/LVPPluginException.class.php';

class LVPPlugin extends AbstractStudipSystemPlugin {
	
	private $view;
	private $access_granted;
	private $display_title;
	public $path;
	public $template_factory;
	public $user;
	public $permissions;
	public $roles;
	
	public function __construct () {
		parent::__construct();
		
		$this->path = $GLOBALS['ABSOLUTE_URI_STUDIP'].str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__));
		$this->setPluginiconname('assets/images/page_edit.png');
		$this->setDisplayType(SYSTEM_PLUGIN_TOOLBAR);
		$this->user = $this->getUser();
		$this->permissions = $this->user->getPermission();
		$this->roles = $this->getRoles();
		//$this->access_granted = ($this->roles['v_admin'] || $GLOBALS['perm']->have_perm('root'));
		$this->access_granted = $GLOBALS['perm']->have_perm('admin');
	
		if ($this->access_granted) {
			$navigation = new PluginNavigation();
			$navigation->setDisplayname(_("berschneidung"));
			$this->setNavigation($navigation);
			
			$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__) . '/templates');
			
			if (strpos('_' . $GLOBALS['unconsumed'], 'Calendar')) {
				$this->view = LVPPluginView::GetView('Calendar', $this);
				$this->display_title = _("berschneidung - Wochenansicht");
			} else {
				$this->view = LVPPluginView::GetView('Overview', $this);
				$this->display_title = _("berschneidung - bersicht");
			}
		}
	}
	
	public function getPluginName () {
	
		return 'Lehrveranstaltungsplanung';
	}
	
	public function getDisplayTitle () {
	
		return $this->display_title;
	}
	
	/**
	 * berprft, ob der Benutzer eingelogt ist. Falls das nicht der Fall ist,
	 * wird eine Fehlermeldung ausgegeben und die Abarbeitung beendet.
	 *
	 */
	private function checkLogin() {
		// Login prfen
		if (!isset($_SESSION['auth']) or !is_object($_SESSION['auth']) or !isset($_SESSION['auth']->auth) or !isset($_SESSION['auth']->auth['uid']) or ($_SESSION['auth']->auth['uid']=='nobody')) {
			StudIPTemplateEngine::showErrorMessage('Sie mssen eingeloggt sein, um dieses Plugin aufrufen zu drfen');
			die;
		}
	}

	/**
	 * berprft, ob der Benutzer V-Admin, oder M-Admin ist
	 *
	 * @return array
	 */
	private function getRoles() {
		$roleperm = array();
		$roles = $this->user->getAssignedRoles();
		// Benutzer ist Admin
		//TODO: das hier anpassen und so.
		if($this->permissions->hasAdminPermission() && !$this->permissions->hasRootPermission()) {
			foreach ($roles as $role) {
				if ($role->rolename == 'V-Admin') {
					$roleperm['v_admin'] = true;
				} elseif($role->rolename == 'M-Admin') {
					$roleperm['m_admin'] = true;
				}
			}
		}
		//Root darf alles!
		elseif($this->permissions->hasRootPermission()) {
			$roleperm['m_admin'] = true;
			$roleperm['v_admin'] = true;
		}
		return $roleperm;
	}
	
	public function initialize () {
		$css = $GLOBALS['ABSOLUTE_URI_STUDIP'] . $this->getPluginPath() . '/assets/LVPPlugin.css';
		$GLOBALS['_include_additional_header'] = '<link rel="stylesheet" href="'.$css.'" type="text/css">';
		foreach (glob(dirname(__FILE__).'/js/*.js') as $js) {
			$GLOBALS['_include_additional_header'] .= '<script src="'.$this->path.'/js/'.basename($js).'" type="text/javascript"></script>'."\n";
		}
	}
	
	private function showNavigation () {
		//Login berprfen
		$this->checkLogin();
		
		if ($this->hasNavigation()) {
			$nav = $this->getNavigation();
			$structure['plugin_' . $this->getPluginid()] = array('topKat' => '', 'name' => $nav->getDisplayname(), 'link' => $nav->getLink(), 'active' => false);
			
			$structure['_plugin_' . $this->getPluginid() . '_overview'] = array('topKat' => 'plugin_' . $this->getPluginId(), 'name' => _("bersicht"), 'link' => PluginEngine::getLink($this, array(), 'ShowOverview'), 'active' => false);
			
		//	$structure['_plugin_' . $this->getPluginid() . '_calendar'] = array('topKat' => 'plugin_' . $this->getPluginId(), 'name' => _("Wochenansicht"), 'link' => PluginEngine::getLink($this, array(), 'ShowCalendar'), 'active' => false);
			
			/*
			$pluginsubmenu["_plugin_" . $this->getPluginId()] = array('topKat' => 'plugin_' . $this->getPluginId(), 'name' => $nav->getDisplayname(), 'link' => PluginEngine::getLink($this, $nav->getLinkParams()), 'active' => false);
			$submenu = $nav->getSubMenu();
			foreach ($submenu as $submenuitem) {
				$pluginsubmenu['plugin_' . $this->getPluginId() . '_' . $submenuitem->getDisplayname()] = array ('topKat' => 'plugin_' . $this->getPluginId(), 'name' => $submenuitem->getDisplayname(), 'link' => $submenuitem->getLink(), 'active' => false);
				foreach ($submenuitem->getLinkParams() as $key => $val) {
					if (isset($_GET[$key]) && $_GET[$key] == $val) {
						$reiter_view = 'plugin_' . $this->getPluginId() . '_' . $submenuitem->getDisplayname();
						break;
					}
			}
			*/
			
			// ermitteln des aktiven Reiters
			if (strpos('_' . $GLOBALS['unconsumed'], 'Calendar')) {
				$reiter_view = '_plugin_' . $this->getPluginid() . '_calendar';
			} else {
				$reiter_view = '_plugin_' . $this->getPluginid() . '_overview';
			}
			
		//	$structure = array_merge((array) $structure, (array) $pluginsubmenu);
			$reiter = new reiter();
			$reiter->create($structure, $reiter_view);
		}
	}
	
	public function actionShow () {
		$this->showNavigation();
		echo $this->view->show();
	}
	
	public function actionShowOverview () {
		$this->actionShow();
	}
	
	public function actionShowCalendar () {
		$this->actionShow();
	}
	
	public function actionOverviewChangeFoS () {
		$this->actionShow();
	}
	
	public function actionOverviewRedirectRaumzeit () {
		$this->actionShow();
	}
	
	public function actionOverviewGetElement () {
		$module_id = substr($_REQUEST['module_id'], 0, 32);
		$seminar_id = substr($_REQUEST['seminar_id'], 0, 32);
		$element_id = substr($_REQUEST['element'], 0, 10);
		if ($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] == $element_id) {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id]);
		} else {
			$_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] = $element_id;
		}
		switch ($element_id) {
			case 'note' :
				$element = $this->view->getNote($module_id, $seminar_id);
				break;
			case 'messages' :
				$element = $this->view->getMessages($module_id, $seminar_id);
				break;
			case 'conflicts' :
				$element = $this->view->getConflicts($module_id, $seminar_id);
				break;
			default:
				$element = '';
				// throw error
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $element;
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewEditNote () {
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getNote($_REQUEST['module_id'], $_REQUEST['seminar_id'], true);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->view->setEdit($_REQUEST['seminar_id']);
			$this->actionShow();
		}
	}
	
	public function actionOverviewUpdateNote () {
		if (LVPPluginHelper::InsertNote($_REQUEST['seminar_id'], $_REQUEST['note'])) {
			$this->view->addMsg('msg', _("Die Notiz wurde gendert."), $_REQUEST['seminar_id']);
		} else {
			$this->view->addMsg('error', _("Die Notiz konnte nicht gendert werden."), $_REQUEST['seminar_id']);
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getNote($_REQUEST['module_id'], $_REQUEST['seminar_id']);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewDeleteNote () {
		if (LVPPluginHelper::DeleteNote($_REQUEST['seminar_id'])) {
			$this->view->addMsg('msg', _("Die Notiz wurde gelscht."), $_REQUEST['seminar_id']);
		} else {
			$this->view->addMsg('error', _("Die Notiz konnte nicht gelscht werden."), $_REQUEST['seminar_id']);
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getNote($_REQUEST['module_id'], $_REQUEST['seminar_id']);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewGetNote () {
		$module_id = substr($_REQUEST['module_id'], 0, 32);
		$seminar_id = substr($_REQUEST['seminar_id'], 0, 32);
		if ($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] == 'note') {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id]);
		} else {
			$_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] = 'note';
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getNote($module_id, $seminar_id);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewGetMessages () {
		$module_id = substr($_REQUEST['module_id'], 0, 32);
		$seminar_id = substr($_REQUEST['seminar_id'], 0, 32);
		if ($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] == 'messages') {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id]);
		} else {
			$_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] = 'messages';
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getMessages($module_id, $seminar_id);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewSendMessage () {
		$error = true;
		if (!isset($_REQUEST['uname']) || !sizeof($_REQUEST['uname'])) {
			$this->view->addMsg('error', _("Bitte whlen Sie einen Epfnger aus."), $_REQUEST['seminar_id']);
		} else if (!trim($_REQUEST['subject'])) {
			$this->view->addMsg('error', _("Bitte geben Sie einen Betreff ein."), $_REQUEST['seminar_id']);
		} else if (!trim($_REQUEST['message'])) {
			$this->view->addMsg('error', _("Bitte geben Sie eine Nachricht ein."), $_REQUEST['seminar_id']);
		} else if (LVPPluginHelper::InsertMessage($_REQUEST['seminar_id'], $_REQUEST['uname'], $_REQUEST['subject'], $_REQUEST['message'])) {
			$this->view->addMsg('msg', _("Die Nachricht wurde verschickt."), $_REQUEST['seminar_id']);
			$error = false;
		} else {
			$this->view->addMsg('error', _("Die Nachricht konnte nicht verschickt werden."), $_REQUEST['seminar_id']);
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getMessages($_REQUEST['module_id'], $_REQUEST['seminar_id'], $error);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->view->setEdit($_REQUEST['seminar_id']);
			$this->actionShow();
		}
	}
	
	public function actionOverviewGetConflicts () {
		$module_id = substr($_REQUEST['module_id'], 0, 32);
		$seminar_id = substr($_REQUEST['seminar_id'], 0, 32);
		$fach_nr = intval($_REQUEST['fach_nr']);
		if ($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] == 'conflicts' . $fach_nr) {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id]);
		} else {
			$_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] = 'conflicts' . $fach_nr;
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getConflicts($module_id, $seminar_id, $fach_nr, $pb);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewGetPbConflicts () {
		$module_id = substr($_REQUEST['module_id'], 0, 32);
		$seminar_id = substr($_REQUEST['seminar_id'], 0, 32);
		if ($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] == 'conflicts_pb') {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id]);
		} else {
			$_SESSION['LVPPlugin']['open_elements'][$module_id . '-' . $seminar_id] = 'conflicts_pb';
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getPbConflicts($module_id, $seminar_id);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewNotifyOpen () {
		list($element_id, $module_seminar_id) = explode('_', $_REQUEST['id']);
		$_SESSION['LVPPlugin']['open_elements'][$module_seminar_id] = $element_id;
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewNotifyClose () {
		list($element_id, $module_seminar_id) = explode('_', $_REQUEST['id']);
		if (isset($_SESSION['LVPPlugin']['open_elements'][$module_seminar_id])) {
			unset($_SESSION['LVPPlugin']['open_elements'][$module_seminar_id]);
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			$element_id = substr($_REQUEST['id'], 0, 80);
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewToggleVisibility () {
		$visible = LVPPluginHelper::ToggleVisibility($_REQUEST['seminar_id']);
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getVisibility($_REQUEST['seminar_id'], $visible);
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewToggleIgnore () {
		$ignored = LVPPluginHelper::GetIgnoredCourses(array($_REQUEST['seminar_id']));
		if (sizeof($ignored)) {
			LVPPluginHelper::DeleteIgnoredCourse($_REQUEST['seminar_id']);
		} else {
			LVPPluginHelper::InsertIgnoredCourse($_REQUEST['seminar_id']);
		}
		if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			ob_end_clean();
			Header('Content-Type: text/html; charset=WINDOWS-1252');
			echo $this->view->getIgnored($_REQUEST['seminar_id'], !isset($ignored[$_REQUEST['seminar_id']]));
			ob_start(create_function('$a', 'return "";'));
		} else {
			$this->actionShow();
		}
	}
	
	public function actionOverviewRefresh () {
		$this->view->setRefresh();
		$this->actionShow();
	}
	
	public function actionOverviewNew () {
		if (($_REQUEST['fachsem'] && !$_REQUEST['add_abschluss']) || ($_REQUEST['add_abschluss'] && $_REQUEST['abschluss_2'])) {
			$this->view->setRefresh();
		}
		$this->actionShow();
	}
	
	public function actionOverviewReset () {
		if (isset($_SESSION['LVPPlugin']['mod_select'])) {
			unset($_SESSION['LVPPlugin']['mod_select']);
		}
		if (isset($_SESSION['LVPPlugin']['mod_select_2'])) {
			unset($_SESSION['LVPPlugin']['mod_select']);
		}
		$_SESSION['LVPPlugin']['show_all'] = false;
		$this->actionShow();
	}
	
	public function actionCalendarShowFoS () {
		$this->actionShow();
	}
	
	public function actionCalendarShowModule () {
		$this->actionShow();
	}
	
	public function actionCalendarShowSingleCourse () {
		$this->actionShow();
	}
	
	public function actionCalendarNext () {
		$atime = $this->view->getTime();
		$this->view->setTime(mktime(12, 0, 0, date('n', $atime), date('j', $atime) + 7), date('Y', $atime));
		$this->actionShow();
	}
	
	public function actionCalendarPrevious () {
		$atime = $this->view->getTime();
		$this->view->setTime(mktime(12, 0, 0, date('n', $atime), date('j', $atime) - 7), date('Y', $atime));
		$this->actionShow();
	}
	
	public function actionCalendarUp () {
		if ($this->view->getDayTime() <= 8) {
			$this->view->setDayTime(0);
		} else if ($this->view->getDayTime() >= 11) {
			$this->view->setDayTime(8);
		} else {
			$this->view->setDayTime(8);
		}
		$this->actionShow();
	}
	
	public function actionCalendarDown () {
		if ($this->view->getDayTime() == 8) {
			$this->view->setDayTime(11);
		} else if ($this->view->getDayTime() == 0) {
			$this->view->setDayTime(8);
		} else {
			$this->view->setDayTime(8);
		}
		$this->actionShow();
	}
	
	public function actionCalendarNextConflict () {
		$this->actionShow();
	}
	
	public function actionCalendarPreviousConflict () {
		$this->actionShow();
	}
	
	public function actionCalendarGoTo () {
		$this->actionShow();
	}
	
	public function actionCalendarSendMessage () {
		$this->actionShow();
	}
	
	public function actionCalendarRedirectRaumzeit () {
		$this->actionShow();
	}

}
?>