<?php
/**
 * StudienmodulManagement.class.php
 *
 * @author Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @version 0.0.1
 */

require_once 'bootstrap.inc.php';

class StudienmodulManagement extends AbstractStudIPCorePlugin implements StudienmodulManagementPlugin
{
	private $tabs = array(
		'catalog'=>'Studienmodule suchen',
	);
	private $template_factory;

	public function __construct()
	{
		parent::__construct();

		$navigation = new PluginNavigation();
		$navigation->setDisplayname(reset($this->tabs));
		$navigation->addLinkParam('action', key($this->tabs));

		/*foreach ($this->tabs as $action=>$tab)
		{
			$subnavigation = new PluginNavigation();
			$subnavigation->setDisplayname(_($tab));
			$subnavigation->addLinkParam('action', $action);
			$navigation->addSubmenu($subnavigation);
		}*/
		$navigation->setPlugin($this);
		$this->module_catalog_navigation = $navigation;

		$tab = new PluginNavigation();
		$tab->addLinkParam('action', 'my_modules');
		$tab->setDisplayname(_('Meine Studienmodule'));
		$tab->setPlugin($this);
		$this->my_modules_navigation = $tab;


		$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
	}

	function getModuleCatalogNavigation()
	{
		return $this->module_catalog_navigation;
	}

	function getMyModulesNavigation()
	{
		return $this->my_modules_navigation;
	}

	function display_action($action)
	{
		$this->$action();
	}


	public function actionShow()
	{
		$action = isset($_REQUEST['action'])
			? $_REQUEST['action']
			: key($this->tabs);

		if (is_callable(array($this, 'renderaction_'.$action)))
		{
			call_user_func(array($this, 'renderaction_'.$action));
		}
		else
		{
			printf("Method '%s' with action '%s'\n", __METHOD__, $action);
		}
	}

	public function renderaction_catalog()
	{
		$current_semester = isset($_REQUEST['semester'])
			? $_REQUEST['semester']
			: ModuleHelper::GetCurrentSemester();

		$sem_tree_id = isset($_REQUEST['node'])
			? $_REQUEST['node']
			: null;

		$config = new Config('SEM_VISIBILITY_PERM');
		$required_permission = $config->getValue();
		$permissions = $this->getUser()->getPermission();
		$omnipotent = empty($required_permissions) or (($required_permission=='root')
			? $permissions->hasRootPermission()
			: ($required_permission=='admin')
			? $permissions->hasAdminPermission()
			: ($required_permission=='dozent')
			? $permissions->hasTeacherPermission()
			: $permissions->hasStudentPermission());

		$sem_tree = new SemTreeWalker($sem_tree_id, $omnipotent);
		$sem_tree->skip_level(2);
		$sem_tree->set_semester(empty($current_semester) ? null : $current_semester);
		$sem_tree->set_fachsemester(empty($_REQUEST['fachsemester']) ? null : $_REQUEST['fachsemester']);
		$nodes = $sem_tree->get_nodes();
		$assets = $sem_tree->get_assets();

		$contents = $sem_tree->get_contents();
		$leaf = $contents['type']==4;

		$path = array();
		while ($sem_tree->get_contents())
		{
			array_unshift($path, $sem_tree->get_contents());
			$sem_tree->up();
		}

		$latest_ids = ModuleDB::GetLatestModuleIds(5);
		$toplist = array();
		foreach ($latest_ids as $id)
		{
			$module = ModuleDB::LoadConcrete($id);
			$title = ModuleHelper::GetModuleName($module);
			array_push($toplist, sprintf('<li><a href="%s" class="module_info">%s%s</a> (%s)</li>',
				ModuleHelper::GetModuleLink($id, $module['semester']),
				substr($title, 0, 40),
				strlen($title)>40?'&hellip;':'',
				date('d.m.Y', $module['letzteAenderung'])
			));
		}

		$infobox = array();

		if ($leaf or !empty($_REQUEST['needle']))
		{
			array_push($infobox, array(
				'kategorie' => _('Informationen:'),
				'eintrag' => array(
					array(
						'icon' => 'report.png',
						'text' => _('Klicken Sie auf das Info-Symbol, um weiterfhrende Informationen zu einem Modul zu erhalten.')
					)
				)
			));
		}

		array_push($infobox, array(
			'kategorie' => _('Aktionen:'),
			'eintrag' => array(
				array(
					'icon' => 'suchen.gif',
					'text' => _('Sie knnen hier nach allen Modulen suchen und sich Informationen sowie die Veranstaltungen eines Moduls anzeigen lassen.')
				)
			)
		));

		$toplist = implode($toplist);
		array_push($infobox, array(
			'kategorie' => _('Topliste:'),
			'eintrag' => array(
				array(
					'icon' => 'blank.gif',
					'text' => '<strong>Neueste Module</strong><ol>'.$toplist.'</ol>',
				),
			),
		));

		$template = $this->template_factory->open('ma_catalog');

		$template->set_attribute('links', array(
			'reset'=>PluginEngine::GetLink($this, array('action'=>'catalog')),
			'submit'=>PluginEngine::GetLink($this, array('action'=>'catalog')),
			'node'=>str_replace(urlencode('%s'), '%s', PluginEngine::GetLink($this, array('node'=>'%s', 'action'=>'catalog', 'semester'=>$current_semester, 'fachsemester'=>@$_REQUEST['fachsemester']))),
			'root'=>PluginEngine::GetLink($this, array('semester'=>$current_semester, 'fachsemester'=>$_REQUEST['fachsemester'], 'action'=>'catalog')),
		));
		$template->set_attribute('infobox', $infobox);
		$template->set_attribute('semester', $current_semester);
		$template->set_attribute('needle', @$_REQUEST['needle']);
		$template->set_attribute('fachsemester', @$_REQUEST['fachsemester']);
		$template->set_attribute('semesters', ModuleHelper::GetSemesters());
		$template->set_attribute('nodes', $nodes);
		$template->set_attribute('path', $path);
		$template->set_attribute('assets', $assets);

		if (!empty($_REQUEST['needle']))
		{
			$results = ModuleDB::SearchModules($_REQUEST['needle'], $_REQUEST['semester'], $_REQUEST['fachsemester']);

			$result_count = empty($current_semester)
				? array_reduce($results, create_function('$v,$w', 'return $v+count($w);'))
				: count($results[$current_semester]);

			$template->set_attribute('result_count', $result_count);
			$template->set_attribute('results', $results);
		}
		elseif ($leaf)
		{
			$nodes = array_filter($nodes, create_function('$a', 'return !empty($a["quantity"]);'));
			$sem_tree_ids = array_map(create_function('$a', 'return $a["sem_tree_id"];'), $nodes);

			$leaf_data = ModuleDB::GetSemestersAndModulesAndSeminareBySemTreeId($sem_tree_ids);

			$template->set_attribute('leaf', $leaf);
			$template->set_attribute('leaf_data', $leaf_data);
		}

		$GLOBALS['CURRENT_PAGE'] = $this->getModuleCatalogNavigation()->getDisplayName();

		include 'lib/include/html_head.inc.php';
		include 'lib/include/header.php';
		include 'lib/include/links_seminare.inc.php';
		echo $template->render();
		include 'lib/include/html_end.inc.php';
		page_close();
	}

	public function renderaction_my_modules()
	{
		$template = $this->template_factory->open($this->user->getPermission()->hasAdminPermission()?'ma_my_modules_admin':'ma_my_modules');

		if (!empty($_REQUEST['modopensem']))
			$_SESSION['modopensem'] = array_unique(array_merge((array)@$_SESSION['modopensem'], array($_REQUEST['modopensem'])));
		elseif (!empty($_REQUEST['modclosesem']))
			$_SESSION['modopensem'] = array_diff((array)@$_SESSION['modopensem'], array($_REQUEST['modclosesem']));
		if (!empty($_POST['semester_id']))
			$_SESSION['modopensem'] = array($_POST['semester_id']);

		$semesters = ModuleHelper::GetSemestersForUser($GLOBALS['auth']->auth['uid']);
		$current_semester = empty($_REQUEST['semester_id'])
			? ModuleHelper::GetCurrentSemester()
			: $_REQUEST['semester_id'];
		if (!isset($_SESSION['modopensem']))
			$_SESSION['modopensem'] = array($current_semester);

		$modules = array();
		if ($this->user->getPermission()->hasAdminPermission())
		{
			$institutes = ModuleHelper::GetInstitutesForUser($this->user->getUserId(), false);
			$institute_id = isset($_REQUEST['institute_id'])
				? $_REQUEST['institute_id']
				: reset($institutes);
			$template->set_attribute('institutes', $institutes);
			$template->set_attribute('institute_id', $institute_id);

			$modules = ModuleDB::GetModulesByInstituteId($institute_id, !empty($_REQUEST['semester_id']) ? $_REQUEST['semester_id'] : null);
			$modules = ModuleDB::LoadConcrete(array_unique($modules));

			$semesters = array();
			$tmp = array();
			foreach ($modules as $module_id => $data)
			{
				array_push($semesters, $data['semester']);
				if (empty($tmp[ $data['semester'] ]))
					$tmp[$data['semester']] = array();
				if (empty($tmp[ $data['semester'] ][ $data['fach_id'] ]))
					$tmp[ $data['semester'] ][ $data['fach_id'] ] = array();
				if (empty($tmp[ $data['semester'] ][ $data['fach_id'] ][ $data['abschluss_id'] ]))
					$tmp[ $data['semester'] ][ $data['fach_id'] ][ $data['abschluss_id'] ] = array();
				array_push($tmp[ $data['semester'] ][ $data['fach_id'] ][ $data['abschluss_id'] ], $data);
			}
			$modules = $tmp;

			$semesters = array_unique($semesters);
			rsort($semesters);
		}
		else
		{
			foreach ($semesters as $semester_id)
				foreach (ModuleHelper::GetSeminarsForUserAndSemester($GLOBALS['auth']->auth['uid'], $semester_id) as $seminar_id)
				{
					$modules = array_merge($modules, ModuleDB::GetModulesForSeminar($seminar_id));
//					break 2;
				}

			$modules = ModuleDB::LoadConcrete(array_unique($modules));
			$tmp = array();
			foreach ($modules as $module_id=>$data)
			{
				if (empty($tmp[$data['semester']]))
					$tmp[$data['semester']] = array();
				$tmp[$data['semester']][$module_id] = $data;
			}
			$modules = $tmp;
		}

		$infobox = array();
		array_push($infobox, array(
			'kategorie' => _('Informationen:'),
			'eintrag' => array(
				array(
					'icon' => 'blank.gif',
					'text' => $this->user->getPermission()->hasAdminPermission()
						? _('Die angezeigten Module ergeben sich aus den Ihnen zugeordneten Einrichtungen.')
						: _('Die angezeigten Module ergeben sich aus Ihren belegten Veranstaltungen.'),
				),
				array(
					'icon' => 'report.png',
					'text' => _('Bei einem Klick auf dieses Icon erhalten Sie weiterfhrende Informationen zu diesem Modul.')
				)
			)
		));

		$template->set_attribute('current_semester', $current_semester);
		$template->set_attribute('user_semesters', $semesters);
		$template->set_attribute('open_semesters', (array)$_SESSION['modopensem']);
		$template->set_attribute('semester_info', ModuleHelper::GetSemesterInfo($semesters));
		$template->set_attribute('modules', $modules);
		$template->set_attribute('infobox', $infobox);
		$template->set_attribute('links', array(
			'module'=>$this->getModuleDescriptionLink('%s', '%s'),
			'open'=>str_replace(urlencode('%s'), '%s', PluginEngine::GetLink($this, array('modopensem'=>'%s', 'institute_id'=>$institute_id, 'action'=>'my_modules'))),
			'close'=>str_replace(urlencode('%s'), '%s', PluginEngine::GetLink($this, array('modclosesem'=>'%s', 'institute_id'=>$institute_id, 'action'=>'my_modules'))),
			'submit' => PluginEngine::GetLink($this, array('action' => 'my_modules')),
		));

		$GLOBALS['CURRENT_PAGE'] = $this->getMyModulesNavigation()->getDisplayName();

		include 'lib/include/html_head.inc.php';
		include 'lib/include/header.php';
		include 'lib/include/links_seminare.inc.php';
		echo $template->render();
		include 'lib/include/html_end.inc.php';
		page_close();
	}


	public function renderaction_show_module()
	{
		$remove_layout = isset($_GET['remove_layout']) || !empty($_SERVER['HTTP_X_REQUESTED_WITH']);

		$module_id = $_GET['id'];
		$semester_id = $_GET['semester'];


		if (isset($_REQUEST['print']))
		{
			if ($module_id{0}=='!')
			{
				$module = ModuleDB::Load(substr($module_id, 1));
			}
			else
			{
				$module = ModuleDB::LoadConcrete($module_id);
				if (empty($module['aktiv']))
				{
					$module = ModuleDB::Load($module['modul_abst_id']);
				}
			}

			$template = $this->template_factory->open('ma_viewmodule');
			$template->set_attribute('print', true);
			$template->set_attribute('show_seminars', !empty($_REQUEST['seminars']));
			$template->set_attribute('module', $module);
			$template->set_attribute('remove_layout', $remove_layout);
			$template->set_attribute('studiengaenge', ModuleDB::GetStudienGaengeForModule($module['modul_abst_id']));
			$template->set_attribute('seminars', ModuleDB::GetSeminarDataForModule($module['modul_abst_id'], $semester_id));
			$template->set_attribute('links', array(
				'seminars'=>PluginEngine::GetLink($this, array('id'=>$module['modul_abst_id'], 'semester'=>$semester_id, 'show_seminars'=>1, 'hide_links'=>!empty($_REQUEST['hide_links']), 'action' => 'show_module')),
			));
		}
		elseif (isset($_REQUEST['show_seminars']))
		{
			$template = $this->template_factory->open('ma_viewseminars');
			$template->set_attribute('via_ajax', !empty($_SERVER['HTTP_X_REQUESTED_WITH']));
			$template->set_attribute('module', ModuleDB::Load($module_id));
			$template->set_attribute('seminars', ModuleDB::GetSeminarDataForModule($module_id, $semester_id));
			$template->set_attribute('links', array(
				'module'=>ModuleHelper::GetModuleLink($_GET['id'], $semester_id, array('hide_links'=>!empty($_REQUEST['hide_links']))),
			));
		}
		else
		{
			$abstract = $module_id{0}=='!';
			if ($abstract)
			{
				$module = ModuleDB::Load(substr($module_id, 1));
			}
			else
			{
				$module = ModuleDB::LoadConcrete($module_id);
				if (empty($module['aktiv']))
				{
					$abstract = true;
					$module = ModuleDB::Load($module['modul_abst_id']);
				}
			}

			$template = $this->template_factory->open('ma_viewmodule');
			$template->set_attribute('abstract', $abstract);
			$template->set_attribute('module', $module);
			$template->set_attribute('remove_layout', $remove_layout);
			$template->set_attribute('studiengaenge', ModuleDB::GetStudienGaengeForModule($abstract?$module['id']:$module['modul_abst_id']));
			$template->set_attribute('links', array(
				'seminars'=>PluginEngine::GetLink($this, array('id'=>$module['modul_abst_id'], 'semester'=>$semester_id, 'show_seminars'=>1, 'hide_links'=>!empty($_REQUEST['hide_links']), 'action' => 'show_module')),
				'print'=>PluginEngine::GetLink($this, array('id'=>$module_id, 'semester'=>$semester_id, 'hide_links'=>!empty($_REQUEST['hide_links']), 'print'=>true, 'seminars'=>0, 'action'=>'show_module')),
			));
		}

		$template->set_attribute('studiengaenge2', array_map(array($this, 'getModuleStudiengang'), DBManager::get()->query("SELECT sem_tree_id FROM mod_zuordnung mz JOIN sem_tree USING(sem_tree_id) WHERE modul_abst_id='{$module['modul_abst_id']}'")->fetchAll(PDO::FETCH_COLUMN)));



		if (!empty($module['deskriptor']['language']) and $module['deskriptor']['language']!='de')
		{
			$old_language = getenv('LANGUAGE');
			putenv('LANGUAGE=en_US');
			setlocale(LC_MESSAGES, "en_US.UTF-8");
			bindtextdomain("module", realpath(dirname(__FILE__).'/locale'));
			textdomain("module");
			bind_textdomain_codeset("module", 'UTF-8');
		}

		$content = $template->render();

		if (!empty($module['deskriptor']['language']) and $module['deskriptor']['language']!='de')
		{
			putenv('LANGUAGE='.$old_language);
		}

		if (isset($_REQUEST['append_script']))
		{
			$content = Assets::script('prototype', 'effects.js').$this->initializeHtmlHead().$content;
		}
		if ($remove_layout or isset($_REQUEST['print']))
		{
			Header('Content-Type: text/html; charset=windows-1252');
			echo $content;
			//echo studip_utf8encode($content);
		}
		else
		{
			$GLOBALS['CURRENT_PAGE'] = _("Studienmodul: "). ModuleHelper::GetModuleName($module);
			include 'lib/include/html_head.inc.php';
			include 'lib/include/header.php';
			echo $content;
			include 'lib/include/html_end.inc.php';
			page_close();
		}
	}

	public function getModuleInfoHTML($module_id, $semester_id = '', $additional_parameters=array())
	{
		return sprintf('(<a href="%s" class="module_info" %s><img src="%s" style="border: 0; vertical-align:middle" title="Modulbeschreibung" /> %s</a>)',
		//neu return $prepend.sprintf('<a href="%s" class="module_info" %s>(<img src="%s" style="border: 0; vertical-align:middle" title="Modulbeschreibung" /> Modulbeschreibung)</a>',
		//alt return $prepend.sprintf('<a href="%s" class="module_info" %s><img src="%s" style="border: 0; align="absmiddle"/></a>',
			ModuleHelper::GetModuleLink(($a=ModuleDB::SearchConcrete($module_id, $semester_id))?$a:'!'.$module_id, $semester_id, $additional_parameters),
			tooltip(kill_format($info), false),
			Assets::Image_Path('report.png'),
			_('Modulbeschreibung')
		);
	}

	public function getModuleTitle($module_id, $semester_id = '')
	{
		$module = ModuleDB::IsConcrete($module_id, $semester_id)
			? ModuleDB::LoadConcrete(ModuleDB::SearchConcrete($module_id, $semester_id))
			: ModuleDB::Load($module_id);

		return $module
			? ModuleHelper::GetFachName($module['fach_id'], $module).' > '.ModuleHelper::GetAbschlussName($module['abschluss_id'], $module).' > '.ModuleHelper::GetModuleName($module)
			: false.'FEHLER!! '.$module_id;
	}

	public function getModuleStudiengang($module_id, $semester_id = '')
	{
		$module = ModuleDB::IsConcrete($module_id, $semester_id)
			? ModuleDB::LoadConcrete(ModuleDB::SearchConcrete($module_id, $semester_id))
			: ModuleDB::Load($module_id);

		$fach = ModuleHelper::GetFachName($module['fach_id'], $module);
		$temp = DBManager::get()->query("SELECT s.name FROM mod_zuordnung m JOIN studiengaenge s ON (m.fach_id=s.studiengang_id) JOIN sem_tree st ON (st.sem_tree_id=m.sem_tree_id) WHERE st.sem_tree_id IS NOT NULL AND '{$module['id']}' IN (modul_abst_id, m.sem_tree_id)")->fetchColumn();
		if ($temp!=$fach)
			$fach .= ' / '.$temp;


		return $module
			? $fach.' > '.ModuleHelper::GetAbschlussName($module['abschluss_id'], $module)
			: false.'FEHLER!! '.$module_id;
	}

	public function getModuleDescription($module_id, $semester_id = null)
	{
		static $js_written=0;
		$add = $js_written++
			? ''
			: $this->initializeHtmlHead();

		$postfix = '';
		$concrete_id = ModuleDB::SearchConcrete($module_id, $semester_id);
		if ($concrete_id and $module = ModuleDB::LoadConcrete($concrete_id) and !empty($module['deskriptor']['untertitel']))
		{
			$postfix = '<br/><small style="font-weight: normal;">'.$module['deskriptor']['untertitel'].'</small>';
		}

		return basename($_SERVER['PHP_SELF'])=='extern.php'
			? $add.$this->getModuleInfoHTML($module_id, $semester_id, array('hide_links'=>true)).$postfix
			: $this->getModuleInfoHTML($module_id, $semester_id);
	}

	public function getModuleDescriptionLink($module_id, $semester_id = null)
	{
		return $this->getModuleDescriptionUrl($module_id, $semester_id);
	}

	public function getModuleDescriptionUrl($module_id, $semester_id = null)
	{
		return PluginEngine::GetUrl($this, array('id' => ($a=ModuleDB::SearchConcrete($module_id, $semester_id))?$a:'!'.$module_id, 'semester' => $semester_id, 'action' => 'show_module'));
	}

	public function getModuleDescriptionList($studienmodule, $semester_id = null)
	{
		$result = '<ul>';
		foreach ($studienmodule as $module)
		{
			$result .= sprintf('<li>%s %s</li>',
				htmlReady($this->getModuleTitle($module->getId(), $semester_id)),
				$this->getModuleInfoHTML($module->getId(), $semester_id)
			);
		}
		$result .= '</ul>';
		return $result;
	}

	function triggerCourseAddedToModule($module_id, $course_id)
	{
		if (empty($_REQUEST['include_all']) or $_REQUEST['include_all']==='false')
			return;

		$db = DBManager::get();

		$seminar = Seminar::GetInstance($course_id);
		$study_areas = $seminar->getStudyAreas();

		$result = $db->query("SELECT sem_tree_id FROM mod_zuordnung WHERE modul_abst_id = (SELECT modul_abst_id FROM mod_zuordnung WHERE sem_tree_id = '$module_id')");

		$mod_study_areas = $result->fetchAll(PDO::FETCH_COLUMN);
		$study_areas = array_merge($study_areas, $mod_study_areas);

		$seminar->setStudyAreas($study_areas);
	}

	function triggerCourseRemovedFromModule($module_id, $course_id)
	{
	}

	function showStudyCourses()
	{
		return false;
	}

	function initializeHtmlHead()
	{
		$add = '<link href="'.$this->getPluginURL(). '/optimizer.php?lightbox.css,studienmodulmanagement.css" type="text/css" rel="stylesheet"/>'."\n";
		$add .= '<script src="'.$this->getPluginURL(). '/optimizer.php?lightbox.js,studienmodulmanagement.js,showmodules.js" type="text/javascript"></script>'."\n";

		$GLOBALS['_include_additional_header'] .= $add;

		return $add;
	}
}
?>