<?php

require_once 'lib/messaging.inc.php';
require_once 'lib/classes/LockRules.class.php';


class LVPPluginHelper {
	
	/**
	 * Enter description here...
	 *
	 * @param string $lehreinheit_id
	 * @return array()
	 */
	public static function GetFaecher ($lehreinheit_id = '') {
		$faecher = array();
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT sem_tree_id, priority, name FROM `sem_tree` WHERE `type`='3' AND `parent_id`=? ORDER BY `priority`");
		$stmt->execute(array($lehreinheit_id));
		while ($row = $stmt->fetch()) {
			$faecher[$row['sem_tree_id']] = $row['name'];
		}
		return $faecher;
	}
	
	/**
	 * Enter description here...
	 *
	 * @param string $fach_id
	 * @return array()
	 */
	public static function GetAbschluesse ($fach_id) {
		$abschluesse = array();
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT sem_tree_id, priority, name FROM `sem_tree` WHERE `type`='4' AND `parent_id`=? ORDER BY `priority`");
		$stmt->execute(array($fach_id));
		while ($row = $stmt->fetch()) {
			$abschluesse[$row['sem_tree_id']] = array('name' => $row['name']);
		}
		return $abschluesse;
	}
	
	public static function GetSemData ($seminar_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT * FROM seminare WHERE seminar_id = ?");
		$stmt->execute(array($seminar_id));
		if ($row = $stmt->fetch()) {
			return $row;
		}
		return false;
	}
	
	public static function GetMessage ($message_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT * FROM plugin_lvp_message WHERE message_id = ? AND (author_id = ? OR recipient_id = ?)");
		$stmt->execute(array($message_id, $GLOBALS['user']->id, $GLOBALS['user']->id));
		if ($row = $stmt->fetch()) {
			return array($row['message_id'] => $row);
		}
		return false;
	}
	
	public static function GetMessagesByCourse ($seminar_id) {
		$messages = array();
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT * FROM plugin_lvp_message WHERE seminar_id = ? AND (author_id = ? OR recipient_id = ?)");
		$stmt->execute(array($seminar_id, $GLOBALS['user']->id, $GLOBALS['user']->id));
		while ($row = $stmt->fetch()) {
			$messages['message_id'] = $row;
		}
		return $messages;
	}
	
	public static function InsertMessage ($seminar_id, $recipients, $subject, $message, $internal = true) {
		$db = DBManager::Get();
		if ($GLOBALS['perm']->have_studip_perm('admin', $seminar_id)) {
			$time = time();
			/*
			$db = DBManager::Get();
			$query = "INSERT INTO plugin_lvp_message (message_id, seminar_id, author_id, recipient_id, subject, message, internal, mkdate VALUES ";
			$inserted = false;
			$values = array();
			foreach ($recipients as $recipient) {
				$message_id = md5(uniqid('plugin_lvp_message'));
				$values[] = sprintf("('%s', '%s', '%s', '%s', '%s', '%s', %s, %s)", $message_id, mysql_real_escape_string($seminar_id), $GLOBALS['user']->id, mysql_real_escape_string($recipient), mysql_real_escape_string($subject), mysql_real_escape_string($message), ($internal ? '1' : '0'), $time);
			}
			if (sizeof($values)) {
				$query .= implode(',', $values);
				$inserted = $db->query($query);
			}
			if ($inserted && $internal) {
			*/
			
			$messaging = new messaging();
			$messaging->insert_message(substr($db->quote(stripslashes($message)), 1, -1), $recipients, $GLOBALS['user']->id, $time, '', '', '', substr($db->quote(stripslashes($subject)), 1, -1), true);
			return true;
		}
		return false;
	}
	
	public static function UpdateMessageVisited ($message_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("UPDATE plugin_lvp_message SET visited = UNIX_TIMESTAMP() WHERE message_id = ?");
		$stmt->execute(array($message_id));
		if ($stmt->rowCount()) {
			return true;
		}
		return false;
	}
	
	public static function DeleteMessage ($message_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("DELETE FROM plugin_lvp_message WHERE message_id = ? AND (author_id = ? OR recipient_id = ?)");
		$stmt->execute(array($message_id, $GLOBALS['user']->id, $GLOBALS['user']->id));
		if ($stmt->rowCount()) {
			return true;
		}
		return false;
	}
	
	public static function GetNote ($seminar_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT * FROM plugin_lvp_note WHERE seminar_id = ? AND author_id = ?");
		$stmt->execute(array($seminar_id, $GLOBALS['user']->id));
		if ($row = $stmt->fetch()) {
			return array($row['seminar_id'] => $row);
		}
		return false;
	}
	
	public static function HasNote ($seminar_ids) {
		$has_note = array();
		if (is_array($seminar_ids) && sizeof($seminar_ids)) {
			$db = DBManager::Get();
			$sem_ids = array();
			foreach ($seminar_ids as $seminar_id) {
				$sem_ids[] = $db->quote($seminar_id);
			}
			$query = sprintf("SELECT seminar_id FROM plugin_lvp_note WHERE seminar_id IN(%s)", implode(",", $sem_ids));
			$result = $db->query($query);
			while ($row = $result->fetch()) {
				$has_note[$row['seminar_id']] = $row['seminar_id'];
			}
		}
		return $has_note;
	}
	
	public static function InsertNote ($seminar_id, $note) {
		if ($GLOBALS['perm']->have_studip_perm('admin', $seminar_id)) {
			$db = DBManager::Get();
			$stmt = $db->prepare("INSERT INTO plugin_lvp_note (seminar_id, author_id, note, mkdate, chdate) VALUES (?, ?, ?, UNIX_TIMESTAMP(), UNIX_TIMESTAMP()) ON DUPLICATE KEY UPDATE note = VALUES(note), chdate = UNIX_TIMESTAMP()");
			$stmt->execute(array($seminar_id, $GLOBALS['user']->id, $note));
			if ($stmt->rowCount()) {
				return true;
			}
		}
		return false;
	}
	
	public static function DeleteNote ($seminar_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("DELETE FROM plugin_lvp_note WHERE seminar_id = ? AND author_id = ?");
		$stmt->execute(array($seminar_id, $GLOBALS['user']->id));
		if ($stmt->rowCount()) {
			return true;
		}
		return false;
	}
	
	public static function ToggleVisibility ($seminar_id) {
		if ($GLOBALS['perm']->have_studip_perm('admin', $seminar_id)) {
			if(!LockRules::Check($all_sem[$i], 'seminar_visibility')) {
				$db = DBManager::Get();
				$stmt = $db->prepare("SELECT visible FROM seminare WHERE seminar_id = ?");
				$stmt->execute(array($seminar_id));
				$stmt2 = $db->prepare("UPDATE seminare SET visible = ? WHERE seminar_id = ?");
				if ($row = $stmt->fetch()) {
					$visible = $row['visible'] ? 0 : 1;
					$stmt2->execute(array($visible, $seminar_id));
					return ($visible);
				}
			}
		}
		return false;
	}
	
	public static function InsertIgnoredCourse ($seminar_id) {
		if ($GLOBALS['perm']->have_studip_perm('admin', $seminar_id)) {
			$db = DBManager::Get();
			$stmt = $db->prepare("INSERT IGNORE INTO plugin_lvp_ignore SET seminar_id = ?, user_id = ?, mkdate = UNIX_TIMESTAMP()");
			$stmt->execute(array($seminar_id, $GLOBALS['user']->id));
			if ($stmt->rowCount()) {
				return true;
			}
		}
		return false;
	}
	
	public static function GetIgnoredCourses ($seminar_ids) {
		$ignored_courses = array();
		if (is_array($seminar_ids) && sizeof($seminar_ids)) {
			$db = DBManager::Get();
			$sem_ids = array();
			foreach ($seminar_ids as $seminar_id) {
				$sem_ids[] = $db->quote($seminar_id);
			}
			$query = sprintf("SELECT * FROM plugin_lvp_ignore WHERE seminar_id IN (%s)", implode(',', $sem_ids));
			$result = $db->query($query);
			while ($row = $result->fetch()) {
				$ignored_courses[$row['seminar_id']] = $row;
			}
		}
		return $ignored_courses;
	}
	
	public function DeleteIgnoredCourse ($seminar_id) {
		if ($GLOBALS['perm']->have_studip_perm('admin', $seminar_id)) {
			$db = DBManager::Get();
			$stmt = $db->prepare("DELETE FROM plugin_lvp_ignore WHERE seminar_id = ?");
			$stmt->execute(array($seminar_id));
			if ($stmt->rowCount()) {
				return true;
			}
		}
		return false;
	}
	
	public function GetAdminsForCourse ($seminar_id) {
		$db = DBManager::Get();
		$stmt = $db->prepare("SELECT username, {$GLOBALS['_fullname_sql']['no_title_short']} AS fullname FROM seminare s LEFT JOIN Institute i1 USING(Institut_id) INNER JOIN user_inst ui ON (ui.Institut_id = i1.Institut_id OR ui.Institut_id = i1.fakultaets_id) INNER JOIN auth_user_md5 aum USING (user_id) WHERE s.seminar_id = ? AND ui.inst_perms = 'admin' AND ui.user_id <> ?");
		$stmt->execute(array($seminar_id, $GLOBALS['user']->id));
		$admins = array();
		while ($row = $stmt->fetch()) {
			$admins[$row['username']] = $row;
		}
		return $admins;
	}
	
	
}

?>