<?php
require_once 'lib/classes/SemesterData.class.php';
require_once 'ModulVerwaltungDB.class.php';
require_once 'LVPPluginHelper.class.php';


class LVPPluginOverlappingCoursesList {
	
	private $user_id;
	private $semester_id;
	private $fachsem_id;
	private $fach_id;
	private $first_fach_id = '';
	private $abschluss_ids = array();
	private $institut_ids;
	private $check_institut_ids = array();
	private $no_fachsem = false;
	private $show_all = false;
	// Alle gefundenen Kurse werden als Kurse des Professionalisierungsbereichs behandelt.
	// Manuelles setzen evtl. erforderlich, wenn keine in Tabelle modul_deskriptor das 
	// Feld level_id nicht gesetzt ist.
	private $is_pb_level;
	
	private $check_sem_types = array(1,4);
	// nur erforderlich bei Einschrnkung auf bestimmte Modullevel (siehe $modul_level_sql)
	private $check_modul_level = array(1);
	// Modullevel des Professionalisierungsbereichs
	private $pb_modul_level = array(4,8);
	
	private $courses = array();
	private $modules = array();
	private $fachsem = array();
	private $fach_abschluss = array();
	private $abschluss = array();
	private $overlapping = array();
	private $ignored_courses = array();
	private $pb_courses = array();
	
	private $pb_times = array();
	
	
	public function __construct($user_id, $semester_id){
		$this->user_id = $user_id;
		$this->semester_id = $semester_id;
		
		// Zeiten des Professionalisierungsbereichs abfragen
		$this->setPbTimes();
		
		// garbage collect (db stored conflicts)
		$this->gcConflicts();
	}
	
	public function setFachsem ($fachsem) {
		$this->fachsem_id = $fachsem;
	}
	
	public function setNoFachsem ($no_fachsem = true) {
		$this->no_fachsem = $no_fachsem;
	}
	
	public function setFach ($fach) {
		$this->fach_id = $fach;
	}
	
	public function addAbschluss ($abschluss) {
		$this->abschluss_ids[] = $abschluss;
	}
	
	public function setAbschluss ($abschluesse) {
		if (is_array($abschluesse)) {
			$this->abschluss_ids = $abschluesse;
		}
	}
	
	public function setInstitute ($institut_ids) {
		$this->institut_ids = $institut_ids;
	}
	
	public function checkInstitute ($institut_ids) {
		$this->check_institut_ids = $institut_ids;
	}
	
	/*
	public function setPbKorridor ($pb_korridor) {
		$this->pb_korridor = $pb_korridor;
	}
	*/
	
	public function setIsPbLevel ($is_pb_level = true) {
		$this->is_pb_level = $is_pb_level;
	}
	
	public function setShowAll ($show_all = true) {
		$this->show_all = $show_all;
	}
	
	private function setPbTimes () {
		$pb_times = explode(';', trim(get_config('LVP_ZEITEN_PB')));
		foreach ($pb_times as $pb_time) {
			if (strlen(trim($pb_time))) {
				list($wday, $start_hour, $start_minute, $end_hour, $end_minute) = explode(',', trim($pb_time));
				$this->pb_times[] = array(
					'start' => $start_hour * 60 + $start_minute + $wday * 10000,
					'end' => $end_hour * 60 + $end_minute + $wday * 10000,
					'time' => array(
						'start_hour' => $start_hour,
						'start_minute' => $start_minute,
						'end_hour' => $end_hour,
						'end_minute' => $end_minute,
						'wday' => $wday
					)
				);
			}
		}
	}
	
	public function getPbTimes () {
		return $this->pb_times;
	}
	
	public function getCourseIds () {
		return array_keys($this->courses);
	}
	
	public function resetSearch () {
		$this->fachsem_id = '';
		$this->fach_id = '';
		$this->abschluss_ids = array();
		$this->institut_ids = array();
		$this->courses = array();
		$this->modules = array();
		$this->fachsem = array();
		$this->abschluss = array();
		$this->check_institut_ids = array();
		$this->no_fachsem = false;
		$this->ignored_courses = array();
	}
	
	public function refreshConflicts () {
		$this->deleteConflictsByUser();
		$this->getIgnoredCourses();
		$this->getOverlapping();
		$this->storeOverlapping();
	}
	
	public function restore() {
		$db = DBManager::get();
		$show_sem_sql1 = ",sd1.name AS startsem,IF(s.duration_time=-1, '"._("unbegrenzt")."', sd2.name) AS endsem ";
		$show_sem_sql2 = "LEFT JOIN semester_data sd1 ON ( start_time BETWEEN sd1.beginn AND sd1.ende)
					LEFT JOIN semester_data sd2 ON ((start_time + duration_time) BETWEEN sd2.beginn AND sd2.ende)";
		$sem_types_sql = " AND s.status IN (" . join(",", $this->check_sem_types). ") ";
		$current_semester = SemesterData::GetInstance()->getSemesterData($this->semester_id);
		$semester_sql = " AND s.start_time <=" . $current_semester['beginn'] . " 
						  AND (" . $current_semester['beginn'] . " <= 
						  (s.start_time + s.duration_time) OR s.duration_time = -1) ";
		// wird nicht verwendet, da keine Einschrnkung auf bestimmte Modullevel
		// $modul_level_sql = " AND md.level_id IN (" . join(",", $this->check_modul_level). ") ";
		
		// Kurse des Professionalisierungsbereichs entsprechend ausweisen
		if ($this->is_pb_level) {
			$pb_modul_level_sql = " 1 AS prof_bereich ";
		} else {
			$pb_modul_level_sql = sprintf(" IF(md.level_id IN (%s), 1, 0) AS prof_bereich ", implode(',', $this->pb_modul_level));
		}
		
		if (is_array($this->institut_ids) && sizeof($institut_ids)) {
			$institute_sql = " s.Institut_id IN ('" . implode("','", $this->institut_ids) . "') AND ";
		}
		if ($this->fach_id) {
			$fach_sql = " AND mz.fach_id = '{$this->fach_id}' ";
		} else {
			$fach_sql = '';
		}
		if (sizeof($this->abschluss_ids)) {
			$abschluss_sql = " AND st.parent_id IN ('" . implode("','", $this->abschluss_ids) . "') ";
		} else {
			$abschluss_sql = '';
		}
		
		if ($this->no_fachsem) {
			$fachsem_sql = ' AND mf.modul_kon_id IS NULL ';
		} else if ($this->fachsem_id) {
			$fachsem_sql = " AND mf.fachsem_id = {$this->fachsem_id} ";
		} else {
			$fachsem_sql = '';
		}
		
		if (sizeof($this->check_institut_ids)) {
			$check_institute_sql = sprintf(", IF(s.Institut_id IN('%s'), 1, 0) AS perm ", implode("','", $this->check_institut_ids));
		} else {
			$check_institute_sql = ', 1 AS perm ';
		}
		
		$query = "SELECT s.Seminar_id, s.Name, "
				. "VeranstaltungsNummer AS number, s.Institut_id as institut_id, status, visible"
				. $check_institute_sql
				. ", m.modul_kon_id, mz.modulschluessel, mz.modultitel, mz.modul_abst_id, mz.fach_id, st.parent_id AS abschluss_id"
				. ", st.priority, mf.fachsem_id, sst.sem_tree_id, md.desk_id, md.bezeichnung, md.level_id, "
				. $pb_modul_level_sql
				. $show_sem_sql1
			//	. " FROM user_inst AS a INNER JOIN seminare AS s USING (Institut_id) "
				. " FROM seminare s "
				. " INNER JOIN seminar_sem_tree sst ON s.Seminar_id = sst.seminar_id "
				. " INNER JOIN sem_tree st ON sst.sem_tree_id = st.sem_tree_id "
				. " INNER JOIN mod_zuordnung mz ON mz.sem_tree_id = sst.sem_tree_id "
				. " INNER JOIN module m ON m.modul_abst_id = mz.modul_abst_id "
				. " LEFT JOIN mod_fachsem mf ON (mf.modul_kon_id = m.modul_kon_id)"
				. " INNER JOIN modul_deskriptor md ON md.desk_id = m.desk_id "
				. $show_sem_sql2
				/**********************************************************/
			//	. " WHERE a.user_id='{$this->user_id}' AND a.inst_perms='admin' "
				. " WHERE "
				. $institute_sql
				. " m.semester = '{$this->semester_id}' "
				. $fach_sql
				. $abschluss_sql
				. $fachsem_sql
			//	. " WHERE m.semester = '{$this->semester_id}' "
				/**********************************************************/
				. $sem_types_sql
				. $semester_sql;
		
		$rs = $db->query($query);
		while($row = $rs->fetch()) {
			$name = $row["Name"];
			if($row["startsem"] != $row["endsem"]) {
				$name .= " (" . $row['startsem']	.  " - " . $row['endsem'] . ")";
			}
			$this->courses[$row['Seminar_id']] = array(
					'seminar_id' => $row['Seminar_id'],
					'name' => $name,
					'number' => $row['number'],
					'institut_id' => $row['institut_id'],
					'status' => $row['status'],
					'visible' => $row['visible'],
					'perm' => $row['perm'],
					'prof_bereich' => $row['prof_bereich'],
					'abschluss_id' => $row['abschluss_id']);
			/*$this->modules[$row['modul_kon_id']] = array('modul_kon_id' => $row['modul_kon_id'],
														 'desk_id' => $row['desk_id'],
														 'modul_abst_id' => $row['modul_abst_id'],
														 'modultitel' => $row['modultitel'],
														 'bezeichnung' => $row['bezeichnung']
														 );
			*/
			$this->modules[$row['modul_kon_id']]['modul_kon_id'] = $row['modul_kon_id'];
			$this->modules[$row['modul_kon_id']]['desk_id'] = $row['desk_id'];
			$this->modules[$row['modul_kon_id']]['modul_abst_id'] = $row['modul_abst_id'];
			$this->modules[$row['modul_kon_id']]['modultitel'] = $row['modultitel'];
			$this->modules[$row['modul_kon_id']]['modulschluessel'] = $row['modulschluessel'];
			$this->modules[$row['modul_kon_id']]['bezeichnung'] = $row['bezeichnung'];
			$this->modules[$row['modul_kon_id']]['courses'][$row['Seminar_id']] = $row['Seminar_id'];
			$this->modules[$row['modul_kon_id']]['sem_tree_id'][$row['sem_tree_id']] = array($row['fach_id'], $row['abschluss_id']);
			$this->modules[$row['modul_kon_id']]['priority'] = $row['priority'];
			$this->abschluss[$row['abschluss_id']]['courses'][$row['Seminar_id']] = $row['Seminar_id'];
			$this->abschluss[$row['abschluss_id']]['modules'][$row['modul_kon_id']] = $row['modul_kon_id'];
// 			$this->fach_courses[$row['fach_id']][$row['Seminar_id']] = $row['Seminar_id'];
			if ($row['fachsem_id']) {
				$this->fachsem[$row['fachsem_id']][$row['modul_kon_id']] = $row['fachsem_id'];
			}
			if ($row['prof_bereich']) {
				$this->pb_courses[$row['Seminar_id']] = false;
			}
		}
		if (is_array($this->fachsem)) {
			ksort($this->fachsem);
		}
		
		/*
		foreach($this->modules as $modul_id => $modul_data){
			$rs = $db->query("SELECT fachsem_id FROM mod_fachsem WHERE modul_kon_id = '{$modul_id}'");
			while($row = $rs->fetch()) {
				$this->fachsem[$row['fachsem_id']][] = $modul_id;
			}
		}
		*/
		/*
		foreach($this->fach_abschluss as $id => $bla){
			list($fach_id, $abschluss_id) = explode('-', $id);
			$rs = $db->query("SELECT notiz from struktur_zuordnung WHERE typ = 3 AND objekt_id='$fach_id' LIMIT 1");
			$fach = $rs->fetchColumn(0);
			$rs = $db->query("SELECT notiz from struktur_zuordnung WHERE typ = 4 AND objekt_id='$abschluss_id' LIMIT 1");
			$abschluss = $rs->fetchColumn(0);
			$this->fach_abschluss[$id]['name'] = $fach . ' ' . $abschluss;
		}
		*/
		
	/*if ($perm->is_fak_admin()) {
			$query = "SELECT s.Seminar_id,IF(s.visible=0,CONCAT(s.Name, ' "._("(versteckt)")."'), s.Name) AS Name"
				. ($show_sem ? $show_sem_sql1 : "")
				. " FROM user_inst a LEFT JOIN Institute b ON(a.Institut_id=b.Institut_id AND b.Institut_id=b.fakultaets_id)
				LEFT JOIN Institute c ON(c.fakultaets_id = b.institut_id AND c.fakultaets_id!=c.institut_id)
				LEFT JOIN seminare s ON(s.institut_id = c.institut_id) "
				. ($show_sem ? $show_sem_sql2 : "")
				. "	WHERE a.user_id='$user->id' AND a.inst_perms='admin' AND NOT ISNULL(b.Institut_id) AND s.Name LIKE '%$search_str%'";
			$db->query($query);
			while($db->next_record()){
				$name = $db->f("Name")
					. ($show_sem ? " (".$db->f('startsem')
					. ($db->f('startsem') != $db->f('endsem') ? " - ".$db->f('endsem') : "")
					. ")" : "");
				$search_result[$db->f("Seminar_id")]=array("type"=>"sem",
														"name"=>$name);
			}
	}*/
		$this->restoreOverlapping();
	}
	
	public function getOwnCourses () {
		$own_courses = array();
		$admin_inst_ids = self::GetInstituteByUser();
		foreach ($this->courses as $course) {
			if (in_array($course['institut_id'], $admin_inst_ids)) {
				$own_courses[$course['seminar_id']] = $course['seminar_id'];
			}
		}
		return $own_courses;
	}
	
	public function getCoursesForModule($modul_id){
		$ret = array();
		$sorter = array();
		// ignorierte Kurse mssen immer ausgegeben werden
		// TODO: ist das noch erforderlich, wenn Konflikte immer angezeigt werden?
		if (!sizeof($this->ignored_courses)) {
			$this->getIgnoredCourses();
		}
		if (isset($this->modules[$modul_id])) {
			foreach ($this->modules[$modul_id]['courses'] as $course_id) {
				// nur Veranstaltungen mit Konflikten ausgeben
				if ($this->show_all || isset($this->overlapping[$course_id]) || isset($this->ignored_courses[$course_id])) {
					$ret[$course_id] = $this->courses[$course_id];
					$sorter[$course_id] = $this->courses[$course_id]['number'];
				}
			}
			array_multisort($sorter, SORT_ASC, SORT_STRING, $ret);
		}
		if (sizeof($ret)) {
			return $ret;
		}
		return false;
	}
	
	public function getFachsemester() {
		if (is_array($this->fachsem)) {
			return array_keys($this->fachsem);
		} else {
			return array();
		}
	}
	
	/*
	public function getModules($abschluss, $fachsem = null) {
		$ret = array();
		$sorter = array();
		if (isset($this->abschluss[$abschluss])) {
			foreach ($this->abschluss[$abschluss]['modules'] as $modul_id) {
				if ($fachsem && !isset($this->fachsem[$fachsem][$modul_id])) continue;
				$name = ($this->modules[$modul_id]['bezeichnung'] ? $this->modules[$modul_id]['bezeichnung'] : $this->modules[$modul_id]['modultitel']);
				$ret[$modul_id] = array('modul_id' => $modul_id, 'name' => $name);
				$sorter[$modul_id] = $name;
			}
			array_multisort($sorter, SORT_DESC, SORT_STRING, $ret);
		}
		return $ret;	
	}
	*/
	
	public function getModules ($abschluss) {
		$modules = array();
		$sorter = array();
		if (isset($this->abschluss[$abschluss])) {
			foreach ($this->abschluss[$abschluss]['modules'] as $modul_id) {
				if ($this->fachsem_id && !isset($this->fachsem[$this->fachsem_id][$modul_id])) continue;
			//	$name = ($this->modules[$modul_id]['bezeichnung'] ? $this->modules[$modul_id]['bezeichnung'] : $this->modules[$modul_id]['modultitel']);
				$name = ($this->modules[$modul_id]['modultitel'] ? $this->modules[$modul_id]['modulschluessel'] . ' ' . $this->modules[$modul_id]['modultitel'] : $this->modules[$modul_id]['bezeichnung']);
				$modules[$modul_id] = array('modul_id' => $modul_id, 'name' => $name);
				$sorter[$modul_id] = $this->modules[$modul_id]['priority'];
			}
			array_multisort($sorter, SORT_ASC, SORT_NUMERIC, $modules);
		}
		return $modules;
	}
		
	
	public function getModuldetails($modul_id){
		return $this->modules[$modul_id];
	}
	
	private function getOverlapping() {
		if (!is_array($this->abschluss[$this->abschluss_ids[0]])) {
			$this->overlapping = array();
			return false;
		}
		$courses_ids1 = array_keys($this->abschluss[$this->abschluss_ids[0]]['courses']);
		$this->overlapping = array();
		$dates1 = $this->getDates($courses_ids1);
		if (sizeof($this->abschluss_ids) == 2 && is_array($this->abschluss[$this->abschluss_ids[1]])) {
			$courses_ids2 = array_keys($this->abschluss[$this->abschluss_ids[1]]['courses']);
			$dates2 = $this->getDates($courses_ids2);
		} else {
			$dates2 = array();
		}
		while ($course_dates = array_pop($dates1)) {
			foreach ($course_dates as $course_date) {
				// Termine der Kurse aus dem ersten Fach gegen Zeiten des Professionalisierungsbereichs
				// prfen, wenn Kurs nicht selber aus PB
				$start = date('G', $course_date['start']) * 60 + date('i', $course_date['start']) + date('w', $course_date['start']) * 10000;
				$end = date('G', $course_date['end']) * 60 + date('i', $course_date['end']) + date('w', $course_date['end']) * 10000;
				foreach ($this->pb_times as $pb_time) {
					if (isset($this->pb_courses[$course_date['seminar_id']])) {
						// der Kurs gehrt zum Professionalisierungsbereich
						// liegt der Kurs auerhalb der Zeiten des Professionalisierungsbereichs?
						if (!($start < $pb_time['end'] && $end > $pb_time['start'])) {
							$this->overlapping[$course_date['seminar_id']][] = array('termin1_id' => $course_date['termin_id'], 'termin2_id' => 'OUT_PB', 'fach_nr' => 0);
						}
					} else {
			//		echo "<br>".$start." --> ".$pb_time['end']."<br>";
						// der Kurs gehrt nicht zum Professionalisierungsbereich
						// liegt der Kurs innerhalb der Zeiten des Professionalisierungsbereichs?
						if ($start < $pb_time['end'] && $end > $pb_time['start']) {
							$this->overlapping[$course_date['seminar_id']][] = array('termin1_id' => $course_date['termin_id'], 'termin2_id' => 'IN_PB', 'fach_nr' => 0);
						}
					}
				}
				if (!isset($this->ignored_courses[$course_date['seminar_id']])) {
					foreach ($dates1 as $course_id => $course2_dates) {
						if (!isset($this->ignored_courses[$course_id])) {
							foreach ($course2_dates as $course2_date) {
								if ($course_date['start'] < $course2_date['end'] && $course_date['end'] > $course2_date['start']) {
									$this->overlapping[$course_id][] = array('termin1_id' => $course2_date['termin_id'], 'termin2_id' => $course_date['termin_id'], 'fach_nr' => 1);
									$this->overlapping[$course_date['seminar_id']][] = array('termin1_id' => $course_date['termin_id'], 'termin2_id' => $course2_date['termin_id'], 'fach_nr' => 1);
								}
							}
						}
					}
					foreach ($dates2 as $course_id2 => $course_dates2) {
						foreach ($course_dates2 as $course_date2) {
							if ($course_date['start'] < $course_date2['end'] && $course_date['end'] > $course_date2['start']) {
								$this->overlapping[$course_id2][] = array('termin1_id' => $course_date2['termin_id'], 'termin2_id' => $course_date['termin_id'], 'fach_nr' => 2);
								$this->overlapping[$course_date['seminar_id']][] = array('termin1_id' => $course_date['termin_id'], 'termin2_id' => $course_date2['termin_id'], 'fach_nr' => 2);
							}
						}
					}
				}
			}
		}
		return true;
	}
	
	/*
	public function getConflictingPbCourses () {
		$course_ids = array_keys($this->pb_courses);
		$dates = $this->getDates($course_ids);
		foreach ($course_ids as $course_id) {
			foreach ($this->pb_times as $pb_time) {
				foreach ($dates[$course_id] as $date) {
					$start = date('G', $date['start']) * 60 + date('i', $date['start']) + date('w', $date['start']) * 10000 + 10000;
					$end = date('G', $date['end']) * 60 + date('i', $date['end']) + date('w', $date['end']) * 10000 + 10000;
					if (!($start - 1 < $pb_time['end'] && $end > $pb_time['start'])) {
						$this->pb_courses[$course_id] = true;
						continue 3;
					}
				}
			}
		}
		return $this->pb_courses;
	}
	*/
	
	private function getDates ($course_ids) {
		$dates = array();
		
		$termin_typ = array();
		foreach ($GLOBALS['TERMIN_TYP'] as $typ_id => $typ) {
			if ($typ['sitzung']) {
				$termin_typ [] = $typ_id;
			}
		}
		
		$db = DBManager::Get();
		$query = sprintf("SELECT range_id, termin_id, date, end_time FROM termine WHERE range_id IN ('%s') AND date_typ IN ('%s')", implode("','", $course_ids), implode("','", $termin_typ));
		$result = $db->query($query);
		while ($row = $result->fetch()) {
			$dates[$row['range_id']][] = array('termin_id' => $row['termin_id'], 'start' => $row['date'], 'end' => $row['end_time'], 'seminar_id' => $row['range_id']);
		}
		return $dates;
	}
	
	private function storeOverlapping () {
		$db = DBManager::Get();
		foreach ($this->overlapping as $seminar_id => $ovl_seminar) {
			foreach ($ovl_seminar as $ovl) {
				$query = sprintf("INSERT IGNORE INTO plugin_lvp_conflict SET user_id = '%s', seminar_id = '%s', termin1_id = '%s', termin2_id = '%s', fach_nr = %s, mkdate = UNIX_TIMESTAMP()", $GLOBALS['user']->id, $seminar_id, $ovl['termin1_id'], $ovl['termin2_id'], $ovl['fach_nr']);
				$db->query($query);
			}
		}
	}
	
	private function restoreOverlapping () {
		$this->overlapping = array();
		$db = DBManager::Get();
		$query = sprintf("SELECT * FROM plugin_lvp_conflict WHERE user_id = '%s' AND seminar_id IN ('%s')", $GLOBALS['user']->id, implode("','", array_keys($this->courses)));
	//	$query = sprintf("SELECT plc.*, t.range_id AS seminar_id FROM plugin_lvp_conflict plc LEFT JOIN termine t ON (plc.termin1_id = t.termin_id) WHERE t.range_id IN ('%s')", implode("','", array_keys($this->courses)));
		$result = $db->query($query);
		while ($row = $result->fetch()) {
			$this->overlapping[$row['seminar_id']][] = $row;
		}
	}
	
	public function getConflictsByCourse ($seminar_id) {
		if (isset($this->overlapping[$seminar_id])) {
			return $this->overlapping[$seminar_id];
		}
		return array();
	}
	
	public function getCountConflictingCourses () {
		$db = DBManager::Get();
		$count = array();
		$query = sprintf("SELECT c1.fach_nr, c1.seminar_id, COUNT(DISTINCT c2.seminar_id) AS count FROM plugin_lvp_conflict c1 INNER JOIN plugin_lvp_conflict c2 ON (c2.termin1_id = c1.termin2_id AND c2.user_id = c1.user_id) WHERE c1.user_id = '%s' GROUP BY c1.fach_nr, c1.seminar_id", $GLOBALS['user']->id);
		$result = $db->query($query);
		while ($row = $result->fetch()) {
			$count[$row['fach_nr']][$row['seminar_id']] = $row['count'];
		}
		return $count;
	}
	
	public function getConflictingPbCourses () {
		$db = DBManager::Get();
		$query = sprintf("SELECT seminar_id, termin2_id, COUNT(termin1_id) AS conflicts FROM plugin_lvp_conflict WHERE seminar_id IN('%s') AND fach_nr = 0 GROUP BY seminar_id", implode("','", array_keys($this->courses)));
		$result = $db->query($query);
		$count = array();
		while ($row = $result->fetch()) {
			$count[$row['termin2_id']][$row['seminar_id']] = $row['conflicts'];
		}
		return $count;
	}
	
	public function getIgnoredCourses () {
		if (sizeof($this->ignored_courses)) {
			return $this->ignored_courses;
		}
		$this->ignored_courses = LVPPluginHelper::GetIgnoredCourses(array_keys($this->courses));
		return $this->ignored_courses;
	}
	
	public static function GetConflictsFromCache ($seminar_ids, $fach_nr = null) {
		if (is_null($fach_nr)) {
			$fach_sql = '';
		} else {
			$fach_sql = sprintf("AND plc.fach_nr = %s", intval($fach_nr));
		}
		
		if ($GLOBALS['auth']->auth['perm'] == 'root') {
			$check_institute_sql = ', 1 AS perm ';
		} else {
			$check_institute_sql = sprintf(", IF(s.Institut_id IN('%s'), 1, 0) AS perm ", implode("','", self::GetInstituteByUser()));
		}
		
		$db = DBManager::Get();
		$query = sprintf("SELECT plc.*, t.range_id AS seminar_id, t.date, t.end_time, t.date_typ, (IF (t.metadate_id IS NULL, 1, 0)) AS single, t.metadate_id, s.Name, s.VeranstaltungsNummer AS number, s.status %s FROM plugin_lvp_conflict plc LEFT JOIN termine t ON (termin2_id = t.termin_id) LEFT JOIN seminare s ON (t.range_id = s.Seminar_id) WHERE plc.user_id = '%s' AND plc.seminar_id IN ('%s') %s ORDER BY date ASC", $check_institute_sql, $GLOBALS['user']->id, implode("','", $seminar_ids), $fach_sql);
		$result = $db->query($query);
		$conflicts = array();
		while ($row = $result->fetch()) {
			$conflicts[$row['seminar_id']][] = $row;
			$sorter[$row['seminar_id']] = $row['number'];
		}
		array_multisort($sorter, SORT_ASC, SORT_STRING, $conflicts);
		return $conflicts;
	}
	
	public static function GetPbConflictsFromCache ($seminar_ids) {
		if ($GLOBALS['auth']->auth['perm'] == 'root') {
			$check_institute_sql = ', 1 AS perm ';
		} else {
			$check_institute_sql = sprintf(", IF(s.Institut_id IN('%s'), 1, 0) AS perm ", implode("','", self::GetInstituteByUser()));
		}
		
		$db = DBManager::Get();
		$query = sprintf("SELECT plc.*, t.date, t.end_time, t.date_typ, (IF (t.metadate_id IS NULL, 1, 0)) AS single, t.metadate_id FROM plugin_lvp_conflict plc LEFT JOIN termine t ON (termin1_id = t.termin_id) WHERE plc.user_id = '%s' AND plc.seminar_id IN ('%s') AND termin2_id IN('IN_PB', 'OUT_PB') ORDER BY date ASC", $GLOBALS['user']->id, implode("','", $seminar_ids));
		$result = $db->query($query);
		$conflicts = array();
		while ($row = $result->fetch()) {
			$conflicts[$row['termin2_id']][] = $row;
		}
		return $conflicts;
	}
	
	private function deleteConflictsByUser () {
		$db = DBManager::Get();
		$query = sprintf("DELETE FROM plugin_lvp_conflict WHERE user_id = '%s'", $GLOBALS['user']->id);
		if ($db->query($query)) {
			return true;
		}
		return false;
	}
	
	private function gcConflicts () {
		// probability in percent
		if ((rand() % 100) < 1) {
			$db = DBManager::Get();
			$current_semester = SemesterData::GetInstance()->getSemesterData($this->semester_id);
			$query = sprintf("DELETE FROM plugin_lvp_conflict WHERE mkdate < %s", mktime(12, 0, 0, date('n'), date('j') - 2, date('Y')));
			if ($db->query($query)) {
				return true;
			}
		}
		return false;
	}
	
	public static function GetInstituteByUser () {
		$institut_ids = array();
		$db = DBManager::Get();
		$query = "SELECT i2.Institut_id FROM user_inst ui LEFT JOIN Institute i1 ON ui.Institut_id = i1.Institut_id INNER JOIN Institute i2 ON i1.Institut_id = i2.fakultaets_id WHERE ui.user_id = '{$GLOBALS['user']->id}' AND ui.inst_perms = 'admin'";
		$result = $db->query($query);
		while ($row = $result->fetch()) {
			$institut_ids[$row['Institut_id']] = $row['Institut_id'];
		}
		return $institut_ids;
	}
	
	public function getDozenten () {
		$dozenten = array();
		if (sizeof($this->courses)) {
			$db = DBManager::Get();
			$query = "SELECT su.seminar_id, {$GLOBALS['_fullname_sql']['no_title_short']} AS fullname FROM seminar_user su LEFT JOIN auth_user_md5 aum USING (user_id) WHERE seminar_id IN ('" . implode("','", array_keys($this->courses)) . "') AND su.status = 'dozent'";
			$result = $db->query($query);
			while ($row = $result->fetch()) {
				$dozenten[$row['seminar_id']][] = $row['fullname'];
			}
		}
		return $dozenten;
	}
	
}

/*
page_open(array('sess' => 'Seminar_Session',
                'auth' => 'Seminar_Default_Auth',
                'perm' => 'Seminar_Perm',
                'user' => 'Seminar_User'));
$test = new OverlapDetectionCoursesList($user->id, '00000000000000000000000000000035');
$test->restore();
foreach ($test->getModules('x0000000000000000000000000000128-x0000000000000000000000000000030') as $modul_id => $module){
	echo '<li>'.$module['name'].'</li><ul>';
	foreach($test->getCoursesForModule($modul_id) as $course_id => $course){
		echo '<li>'.$course['name'].'</li>';
	}
	echo '</ul>';
}
*/
?>