<?php

require_once dirname(__FILE__) . '/LVPPluginHelper.class.php';
require_once dirname(__FILE__) . '/ModulVerwaltungDB.class.php';
require_once dirname(__FILE__) . '/LVPPluginOverlappingCoursesList.class.php';
require_once 'lib/classes/SemesterData.class.php';

class LVPPluginViewOverview extends LVPPluginView {
	private $overlap_detection;
	private $final_degree_id;
	private $edit_note = '';
	private $edit_element = '';
	private $refresh = false;
	
	public function __construct () {
		$sem_data = SemesterData::GetInstance();
		// Semesterauswahl
		if (!isset($_SESSION['LVPPlugin']['current_semester'])) {
			// Veranstaltungen aus dem nchsten Semester als Vorgabe
			$semester = SemesterData::GetInstance()->getNextSemesterData();
			$_SESSION['LVPPlugin']['current_semester'] = $semester['semester_id'];
		} else if (isset($_REQUEST['sem_select'])) {
			$_SESSION['LVPPlugin']['current_semester'] = substr($_REQUEST['sem_select'], 0, 50);
		}
		if (!isset($_SESSION['LVPPlugin']['show_only_conflicts'])) {
			$_SESSION['LVPPlugin']['show_only_conflicts'] = true;
		}
		$this->semester = $sem_data->getNextSemesterData();
		$this->overlap_detection = new LVPPluginOverlappingCoursesList($GLOBALS['user']->id, $_SESSION['LVPPlugin']['current_semester']);
		if (!isset($_SESSION['LVPPlugin']['open_elements'])) {
			$_SESSION['LVPPlugin']['open_elements'] = array();
		}
	}
	
	public function setEdit ($element_id) {
		$this->edit_element = $element_id;
	}
	
	private function isEdit($element_id) {
		return $this->edit_element == $element_id;
	}
	
	public function setRefresh () {
		$this->refresh = true;
	}
	
	public function show () {
		$complete = $this->showAbschlussSelectArea();
		if ($complete) {
			$this->showOverview();
		}
	}
	
	private function showAbschlussSelectArea() {
		$template = $this->plugin->template_factory->open('abschluss_select.tmpl.php');
		$__complete = false;
		$new_overview = false;
		
		if ($_REQUEST['action'] == 'select') {
			unset($_SESSION['LVPPlugin']['mod_select']);
			unset($_SESSION['LVPPlugin']['mod_select_2']);
			$_SESSION['LVPPlugin']['open_elements'] = array();
			if (!isset($_REQUEST['show_only_conflicts'])) {
				$_SESSION['LVPPlugin']['show_only_conflicts'] = false;
			} else {
				$_SESSION['LVPPlugin']['show_only_conflicts'] = true;
			}
		}
		
		$template->set_attribute('sem_selector', SemesterData::GetSemesterSelector(array('name' => 'sem_select', 'class' => 'onchange_submit_lvp'), $_SESSION['LVPPlugin']['current_semester'], 'semester_id', false));
		
		$lehreinheit = empty($_REQUEST['lehreinheit']) ? $_SESSION['LVPPlugin']['mod_select']['current_lehreinheit'] : $_REQUEST['lehreinheit'];
		$fach = empty($_REQUEST['fach']) ? $_SESSION['LVPPlugin']['mod_select']['current_fach'] : $_REQUEST['fach'];
		$abschluss = empty($_REQUEST['abschluss']) ? $_SESSION['LVPPlugin']['mod_select']['current_abschluss'] : $_REQUEST['abschluss'];
		$fachsemester = empty($_REQUEST['fachsem']) ? $_SESSION['LVPPlugin']['mod_select']['current_fachsemester'] : $_REQUEST['fachsem'];
		
		$lehreinheiten = ModulVerwaltungDB::getLehreinheiten($GLOBALS['user']->id, $GLOBALS['perm']->have_perm('root'), true);
		
		//Lehreinheit ausgewhlt
		if (!empty($lehreinheit) && $lehreinheit{0} != 'x') {
			$current_lehreinheit = trim($lehreinheit);
			$template->set_attribute('current_lehreinheit', $current_lehreinheit);
			$faecher = ModulVerwaltungDB::getFaecher($current_lehreinheit);
			$template->set_attribute('faecher', $faecher);
			$template->set_attribute('show_reset', true);
		}
		
		//Fach ausgewhlt
		if (!empty($fach) && isset($faecher) && in_array($fach, array_keys($faecher))) {
			$current_fach = trim($fach);
			$template->set_attribute('current_fach', $current_fach);
			if ($lehreinheit == $current_lehreinheit) {
				$abschluesse = ModulVerwaltungDB::getAbschluesse($current_fach);
				$template->set_attribute('abschluesse', $abschluesse);
			}
		}
		
		//Abschluss ausgewhlt
		if (!empty($abschluss) && isset($abschluesse) && in_array($abschluss, array_keys($abschluesse))) {
			$current_abschluss = trim($abschluss);
			$template->set_attribute('current_abschluss', $current_abschluss);
			if ($fach == $current_fach) {
				$this->overlap_detection->resetSearch();
				$this->overlap_detection->setInstitute($this->overlap_detection->getInstituteByUser());
				$this->overlap_detection->setAbschluss(array($current_abschluss));
				$this->overlap_detection->restore();
				$fachsems = $this->overlap_detection->getFachsemester();
				// berprfung ob Fachsemester vorhanden sind
				// wird hier nicht mehr bentigt, da alle Module angezeigt werden
				/*
				if (sizeof($fachsems)) {
					$template->set_attribute('fachsemester', $fachsems);
					$template->set_attribute('show_button', true);
				} else {
					$template->set_attribute('no_fachsem', true);
				}
				*/
				$template->set_attribute('fachsemester', $fachsems);
				$template->set_attribute('show_button', true);
			}
		}
		
		//Semesterauswahl
		if(!empty($fachsemester)) {
			$current_fachsemester = intval($fachsemester);
			$template->set_attribute('current_fachsemester', $current_fachsemester);
			$_SESSION['LVPPlugin']['mod_select'] = array(
				'current_fachsemester' => $current_fachsemester,
				'current_lehreinheit' => $current_lehreinheit,
				'current_fach' => $current_fach,
				'current_abschluss' => $current_abschluss,
				'current_lehreinheit_name' => $lehreinheiten[$current_lehreinheit],
				'current_fach_name' => $faecher[$current_fach],
				'current_abschluss_name' => $abschluesse[$current_abschluss]['name']);
			if ($abschluss == $current_abschluss) {
				$__complete = true;
				$template->set_attribute('complete', true);
				$template->set_attribute('href_refresh', PluginEngine::getLink($this->plugin, array(), 'OverviewRefresh'));
			}
		}
		
		$template->set_attribute('lehreinheiten', $lehreinheiten);
		
		if ($__complete && ($_REQUEST['add_abschluss'] || isset($_SESSION['LVPPlugin']['mod_select_2']))) {
			$__complete = false;
			$lehreinheiten = ModulVerwaltungDB::getLehreinheiten($GLOBALS['user']->id, true, true);
			$template->set_attribute('add_abschluss', true);
		
			$lehreinheit_2 = empty($_REQUEST['lehreinheit_2']) ? $_SESSION['LVPPlugin']['mod_select_2']['current_lehreinheit'] : $_REQUEST['lehreinheit_2'];
			$fach_2 = empty($_REQUEST['fach_2']) ? $_SESSION['LVPPlugin']['mod_select_2']['current_fach'] : $_REQUEST['fach_2'];
			$abschluss_2 = empty($_REQUEST['abschluss_2']) ? $_SESSION['LVPPlugin']['mod_select_2']['current_abschluss'] : $_REQUEST['abschluss_2'];
			//$fachsemester_2 = empty($_REQUEST['fachsem_2']) ? $_SESSION['LVPPlugin']['mod_select_2']['current_fachsemester'] : $_REQUEST['fachsem_2'];
			
			//Lehreinheit ausgewhlt
			if (!empty($lehreinheit_2) && $lehreinheit_2{0} != 'x') {
				$current_lehreinheit_2 = trim($lehreinheit_2);
				$template->set_attribute('current_lehreinheit_2', $current_lehreinheit_2);
				$faecher_2 = ModulVerwaltungDB::getFaecher($current_lehreinheit_2);
				$template->set_attribute('faecher_2', $faecher_2);
			}
			
			//Fach ausgewhlt
			if (!empty($fach_2) && isset($faecher_2) && in_array($fach_2, array_keys($faecher_2))) {
				$current_fach_2 = trim($fach_2);
				$template->set_attribute('current_fach_2', $current_fach_2);
				if ($lehreinheit_2 == $current_lehreinheit_2) {
					$abschluesse_2 = ModulVerwaltungDB::getAbschluesse($current_fach_2);
					if (sizeof($abschluesse_2) == 1 && $current_fach_2 == $current_fach) {
						$this->addMsg('error', _("Keine sinnvolle Kombination mglich"));
						$this->showMsg();
					} else {
						$template->set_attribute('abschluesse_2', $abschluesse_2);
						$template->set_attribute('show_button_2', true);
					}
				}
			}
			
			//Abschluss ausgewhlt
			if (!empty($abschluss_2) && isset($abschluesse_2) && in_array($abschluss_2, array_keys($abschluesse_2))) {
				$current_abschluss_2 = trim($abschluss_2);
				$template->set_attribute('current_abschluss_2', $current_abschluss_2);
				if ($fach_2 == $current_fach_2) {
					$_SESSION['LVPPlugin']['mod_select_2'] = array(
						'current_lehreinheit' => $current_lehreinheit_2,
						'current_fach' => $current_fach_2,
						'current_abschluss' => $current_abschluss_2,
						'current_lehreinheit_name' => $lehreinheiten[$current_lehreinheit_2],
						'current_fach_name' => $faecher_2[$current_fach_2],
						'current_abschluss_name' => $abschluesse_2[$current_abschluss_2]['name']);
					$__complete = true;
					$template->set_attribute('complete', true);
					$template->set_attribute('href_refresh', PluginEngine::getLink($this->plugin, array(), 'OverviewRefresh'));
				//	$template->set_attribute('action', PluginEngine::getLink($this->plugin, array(), 'OverviewNew'));
				}
			}
			
			$template->set_attribute('lehreinheiten_2', $lehreinheiten);
			
		} else {
			$template->set_attribute('add_abschluss', false);
		}
		
		$template->set_attribute('show_only_conflicts', $_SESSION['LVPPlugin']['show_only_conflicts']);
		$template->set_attribute('href_reset', PluginEngine::getLink($this->plugin, array(), 'OverviewReset'));
		
		$template->set_attribute('action', PluginEngine::getLink($this->plugin, array(), 'OverviewNew'));
		
		//Ausgabe
		echo $template->render();
		
		return $__complete;
	}
	
	function showOverview () {
		$template = $this->plugin->template_factory->open('overview.tmpl.php');
		// collect all courses for given abschluss and fachsemester
		$this->overlap_detection->resetSearch();
		
		$this->overlap_detection->setAbschluss(array($_SESSION['LVPPlugin']['mod_select']['current_abschluss']));
		// zweiter Abschluss ausgewhlt
		if (is_array($_SESSION['LVPPlugin']['mod_select_2'])) {
			$this->overlap_detection->addAbschluss($_SESSION['LVPPlugin']['mod_select_2']['current_abschluss']);
		}
		
		// nur Module ausgeben, die keinem Fachsemester zugeordnet sind
		if ($_SESSION['LVPPlugin']['mod_select']['current_fachsemester'] == -2) {
			$this->overlap_detection->setNoFachsem();
		}
		// alle Fachsemester anzeigen
		else if ($_SESSION['LVPPlugin']['mod_select']['current_fachsemester'] != -1) {
			$this->overlap_detection->setFachsem($_SESSION['LVPPlugin']['mod_select']['current_fachsemester']);
		}
		// nur root hat administrativen Zugriff auf alle gefundenen Veranstaltungen
		// fr alle Anderen werden mit einem Flag versehen
		if (!$GLOBALS['perm']->have_perm('root')) {
			$this->overlap_detection->checkInstitute($this->overlap_detection->getInstituteByUser());
		}
		// auch Veranstaltungen ohne Konflikt anzeigen
		if (!$_SESSION['LVPPlugin']['show_only_conflicts']) {
			$this->overlap_detection->setShowAll();
		}
			
		// Daten abfragen
		$this->overlap_detection->restore();
		
		// wurde der Refresh-Button geklickt, dann Terminkonflikte neu berechnen
		if ($this->refresh) {
			$this->overlap_detection->refreshConflicts();
		}
		
		$modules_abschluss = $this->overlap_detection->getModules($_SESSION['LVPPlugin']['mod_select']['current_abschluss']);
		
		$abschluesse = array(array(
			'id'          => $_SESSION['LVPPlugin']['mod_select']['current_abschluss'],
			'id_abschluss_2' => $_SESSION['LVPPlugin']['mod_select_2']['current_abschluss'],
			'lehreinheit' => $_SESSION['LVPPlugin']['mod_select']['current_lehreinheit_name'],
			'fach'        => $_SESSION['LVPPlugin']['mod_select']['current_fach_name'],
			'abschluss'   => $_SESSION['LVPPlugin']['mod_select']['current_abschluss_name'],
			'modules'     => $this->getModulesData($modules_abschluss, 1)));
		if (is_array($_SESSION['LVPPlugin']['mod_select_2'])) {
			$modules_abschluss = $this->overlap_detection->getModules($_SESSION['LVPPlugin']['mod_select_2']['current_abschluss']);
			
			$abschluesse[] = array(
				'id'          => $_SESSION['LVPPlugin']['mod_select_2']['current_abschluss'],
				'lehreinheit' => $_SESSION['LVPPlugin']['mod_select_2']['current_lehreinheit_name'],
				'fach'        => $_SESSION['LVPPlugin']['mod_select_2']['current_fach_name'],
				'abschluss'   => $_SESSION['LVPPlugin']['mod_select_2']['current_abschluss_name'],
				'modules'     => $this->getModulesData($modules_abschluss, 2));
		}
		
		$template->set_attribute('fachsemester', $_SESSION['LVPPlugin']['mod_select']['current_fachsemester']);
		$template->set_attribute('abschluesse', $abschluesse);
		$template->set_attribute('image_path', $this->plugin->getPluginPath());
		$template->set_attribute('modules', $modules);
		$template->set_attribute('dozenten', $this->overlap_detection->getDozenten());
		$template->set_attribute('conflicts', $this->overlap_detection->getCountConflictingCourses());
		$template->set_attribute('pb_course_conflicts', $this->overlap_detection->getConflictingPbCourses());
		$template->set_attribute('plugin_path', $this->plugin->path);
		$template->set_attribute('has_note', LVPPluginHelper::HasNote($this->overlap_detection->getCourseIds()));
		
		echo $template->render();
		
	}
	
	private function getModulesData ($modules_abschluss, $fach_nr) {
		$modules = array();
		foreach ($modules_abschluss as $m_id => $module) {
			// es werden nur Module mit Veranstaltungen angezeigt
			if (!$courses = $this->overlap_detection->getCoursesForModule($m_id)) {
				continue;
			}
			$modules[$m_id]['name'] = $module['name'];
			$ignored_courses = $this->overlap_detection->getIgnoredCourses();
			foreach ($courses as $seminar_id => $course) {
				$modules[$m_id]['courses'][$seminar_id] = array(
					'number'        => $course['number'],
					'name'          => $course['name'],
					'status'        => $course['status'],
					'visible'       => $course['visible'],
					'perm'          => $course['perm'],
				
					'href_note'     => PluginEngine::getLink($this->plugin, array('module_id' => $m_id, 'seminar_id' => $seminar_id), 'OverviewGetNote'),
					
					'href_messages' => PluginEngine::getLink($this->plugin, array('module_id' => $m_id, 'seminar_id' => $seminar_id), 'OverviewGetMessages'),
					
					'href_conflicts_pb' => PluginEngine::getLink($this->plugin, array('module_id' => $m_id, 'seminar_id' => $seminar_id), 'OverviewGetPbConflicts'),
					
					'href_conflicts' => PluginEngine::getLink($this->plugin, array('module_id' => $m_id, 'fach_nr' => $fach_nr, 'seminar_id' => $seminar_id), 'OverviewGetConflicts'),
					
					'href_conflicts_2' => PluginEngine::getLink($this->plugin, array('module_id' => $m_id, 'fach_nr' => 2, 'seminar_id' => $seminar_id), 'OverviewGetConflicts'),
					
					'subpart_visible' => $this->getVisibility($seminar_id, $course['visible'], $course['perm']),
					
					'href_toggle_visibility' => PluginEngine::getLink($this->plugin, array('seminar_id' => $seminar_id), 'OverviewToggleVisibility'),
					
					'subpart_ignored' => $this->getIgnored($seminar_id, isset($ignored_courses[$seminar_id]), $course['perm']),
					
					'href_toggle_ignore' => PluginEngine::getLink($this->plugin, array('seminar_id' => $seminar_id), 'OverviewToggleIgnore'),
					
					'href_raumzeit' => URLHelper::getLink('raumzeit.php', array('cid' => $seminar_id, 'dates_data' => 'all', 'rzsem_data' => $seminar_id)));
					
				if (isset($_SESSION['LVPPlugin']['open_elements'][$m_id . '-' . $seminar_id])) {
					$element_id = $_SESSION['LVPPlugin']['open_elements'][$m_id . '-' . $seminar_id];
					$modules[$m_id]['courses'][$seminar_id]['element_id'] = $element_id;
					switch ($element_id) {
						case 'note' :
							$modules[$m_id]['courses'][$seminar_id]['element'] = $this->getNote($m_id, $seminar_id);
							break;
						case 'messages' :
							$modules[$m_id]['courses'][$seminar_id]['element'] = $this->getMessages($m_id, $seminar_id);
							break;
						case 'conflicts_pb' :
							$modules[$m_id]['courses'][$seminar_id]['element'] = $this->getPbConflicts($m_id, $seminar_id);
							break;
						case 'conflicts1' : 
							$modules[$m_id]['courses'][$seminar_id]['element'] = $this->getConflicts($m_id, $seminar_id, 1);
							break;
						case 'conflicts2' : 
							$modules[$m_id]['courses'][$seminar_id]['element'] = $this->getConflicts($m_id, $seminar_id, 2);
							break;
						default :
							$modules[$m_id]['courses'][$seminar_id]['element'] = '';
					}
				}
			}
		}
		return $modules;
	}
	
	public function getNote ($module_id, $seminar_id, $edit = false) {
		$template = $this->plugin->template_factory->open('note.tmpl.php');
		
		if ($note = LVPPluginHelper::GetNote($seminar_id)) {
			$template->set_attribute('note', $note[$seminar_id]);
			$template->set_attribute('href_delete', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewDeleteNote'));
		} else {
			$template->set_attribute('no_note', true);
		}
		if ($edit || $this->isEdit($seminar_id)) {
			$template->set_attribute('edit', true);
			$template->set_attribute('href_update', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewUpdateNote'));
			$template->set_attribute('href_cancel', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewGetNote'));
		}
		$template->set_attribute('href_edit', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewEditNote'));
		$template->set_attribute('messages', $this->getMsg($seminar_id));
		$template->set_attribute('plugin_path', $this->plugin->path);
		$template->set_attribute('seminar_id', $seminar_id);
		$template->set_attribute('module_id', $module_id);
		$template->set_attribute('href_close', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id, 'foo' => rand()), 'OverviewGetNote') . "#$module_id-$seminar_id");
		
		return $template->render();
	}
	
	public function getMessages ($module_id, $seminar_id, $edit = false) {
		$template = $this->plugin->template_factory->open('messaging.tmpl.php');
		
		if ($edit || $this->isEdit($seminar_id)) {
			$template->set_attribute('uname_selected', $_REQUEST['uname']);
			$template->set_attribute('subject', stripslashes($_REQUEST['subject']));
			$template->set_attribute('message', stripslashes($_REQUEST['message']));
		} else {
			$template->set_attribute('uname_selected', array());
			$template->set_attribute('subject', '');
			$template->set_attribute('message', '');
		}
		
		$template->set_attribute('module_id', $module_id);
		$template->set_attribute('seminar_id', $seminar_id);
		$template->set_attribute('admins', LVPPluginHelper::GetAdminsForCourse($seminar_id));
		$template->set_attribute('sem_data', LVPPluginHelper::GetSemData($seminar_id));
		$template->set_attribute('href_close', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewGetMessages'));
		$template->set_attribute('href_send', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id), 'OverviewSendMessage'));
		$template->set_attribute('messages', $this->getMsg($seminar_id));
		
		return $template->render();
	}
	
	public function getConflicts ($module_id, $seminar_id, $fach_nr) {
		$template = $this->plugin->template_factory->open('conflicts.tmpl.php');
		
		$template->set_attribute('fach_nr', intval($fach_nr));
		$course_conflicts = LVPPluginOverlappingCoursesList::GetConflictsFromCache(array($seminar_id), $fach_nr);
		foreach ($course_conflicts as $sem_id => $conflicts) {
		//	if ($conflicts['abschluss_id'] == $abschluss_id) {
			$courses[$sem_id]['conflicts_single'] = array();
			$courses[$sem_id]['conflicts_meta'] = array();
			$courses[$sem_id]['href_raumzeit'] = URLHelper::getLink('raumzeit.php', array('cid' => $sem_id, 'dates_data' => 'all', 'rzsem_data' => $sem_id));
			foreach ($conflicts as $conflict) {
				$courses[$sem_id]['title'] = $conflict['Name'];
				$courses[$sem_id]['number'] = $conflict['number'];
				$courses[$sem_id]['status'] = $conflict['status'];
				$courses[$sem_id]['perm'] = $conflict['perm'];
				if ($conflict['single']) {
					$courses[$sem_id]['conflicts_single'][] = array(
						'start' => $conflict['date'],
						'end' => $conflict['end_time']
					);
				} else {
					++$courses[$sem_id]['count_meta'][$conflict['metadate_id']];
					$courses[$sem_id]['conflicts_meta'][$conflict['metadate_id']] = array(
						'start' => $conflict['date'],
						'end' => $conflict['end_time'],
						'metadate_id' => $conflict['metadate_id']
					);
				}
			}
		//}
		}
		
		$template->set_attribute('courses', $courses);
		$template->set_attribute('href_close', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'fach_nr' => $fach_nr, 'seminar_id' => $seminar_id, 'foo' => rand()), 'OverviewGetConflicts') . "#$module_id-$seminar_id");
		$template->set_attribute('times', getRegularOverview($seminar_id, true));
		
		return $template->render();
	}
	
	public function getPbConflicts ($module_id, $seminar_id) {
		$template = $this->plugin->template_factory->open('pb_conflicts.tmpl.php');
		
		$pb_conflicts = LVPPluginOverlappingCoursesList::GetPbConflictsFromCache(array($seminar_id));
		$count = 0;
		foreach ($pb_conflicts as $pb_id => $conflicts) {
			$count++;
			$pbc[$pb_id]['conflicts_single'] = array();
			$pbc[$pb_id]['conflicts_meta'] = array();
			foreach ($conflicts as $conflict) {
				if ($conflict['single']) {
					$pbc[$pb_id]['conflicts_single'][] = array(
						'start' => $conflict['date'],
						'end' => $conflict['end_time']
					);
				} else {
					++$pbc[$pb_id]['count_meta'][$conflict['metadate_id']];
					$pbc[$pb_id]['conflicts_meta'][$conflict['metadate_id']] = array(
						'start' => $conflict['date'],
						'end' => $conflict['end_time'],
						'metadate_id' => $conflict['metadate_id']
					);
				}
			}
		}
		
		$template->set_attribute('pb_conflicts', $pbc);
		$template->set_attribute('href_close', PluginEngine::getLink($this->plugin, array('module_id' => $module_id, 'seminar_id' => $seminar_id, 'foo' => rand()), 'OverviewGetConflicts') . "#$module_id-$seminar_id");
		$template->set_attribute('times', getRegularOverview($seminar_id, true));
		$template->set_attribute('pb_times', $this->overlap_detection->getPbTimes());
		$template->set_attribute('count_conflicts', $count);
		
		return $template->render();
	}
	
	public function getMessageDisplay ($message_id) {
	
	}
	
	public function getVisibility ($seminar_id, $visible, $perm = true) {
		$template = $this->plugin->template_factory->open('visible.tmpl.php');
		
		$template->set_attribute('perm', $perm);
		$template->set_attribute('visible', $visible);
		$template->set_attribute('seminar_id', $seminar_id);
		$template->set_attribute('plugin_path', $this->plugin->path);
		
		return $template->render();
	}
	
	public function getIgnored ($seminar_id, $ignored, $perm = true) {
		$template = $this->plugin->template_factory->open('ignored.tmpl.php');
		
		$template->set_attribute('perm', $perm);
		$template->set_attribute('ignored', $ignored);
		$template->set_attribute('seminar_id', $seminar_id);
		$template->set_attribute('plugin_path', $this->plugin->path);
		
		return $template->render();
	}
	
}

?>