<?php
class SemTreeWalker
{
	private $sem_tree_id;
	private $semester_id = null;
	private $fachsemester = null;
	private $omnipotent = false;
	private $levels_to_skip = array();

	private $level_filter = array(
		1 => '$a["name"] = DBManager::Get()->query("SELECT Name FROM Institute WHERE Institut_id=\'{$a["studip_object_id"]}\'")->fetchColumn(); return $a;',
		3 => '$a["name"] = preg_replace("/^Fach\s+/", "", $a["name"]); return $a;',
		4 => '$a["name"] = preg_replace("/^Abschluss\s+/", "", $a["name"]); return $a;',
	);

	public function __construct($sem_tree_id = null, $omnipotent = false)
	{
		$this->sem_tree_id = is_null($sem_tree_id)
			? 'root'
			: $sem_tree_id;

		foreach ($this->level_filter as $level=>$code)
			$this->level_filter[$level] = create_function('$a', $code);

		$this->omnipotent = $omnipotent;
	}

	public function skip_level($level)
	{
		array_push($this->levels_to_skip, $level);
		$this->levels_to_skip = array_unique($this->levels_to_skip);
	}

	public function set_semester($semester_id)
	{
		$this->semester_id = $semester_id;
	}

	public function set_fachsemester($fachsemester)
	{
		$this->fachsemester = $fachsemester;
	}

	public function get_contents()
	{
		$contents = DBManager::Get()->query("SELECT sem_tree_id, parent_id, name, type, studip_object_id FROM sem_tree WHERE sem_tree_id='{$this->sem_tree_id}'")->fetch();

		if (isset($this->level_filter[ $contents['type'] ]))
			$contents = $this->level_filter[ $contents['type'] ]($contents);

		return empty($contents)
			? null
			: $contents;
	}

	public function get_nodes()
	{
		$result = DBManager::Get()->query("SELECT sem_tree_id, name, type, studip_object_id FROM sem_tree WHERE parent_id = '{$this->sem_tree_id}' AND type<>0 GROUP BY sem_tree_id ORDER BY name ASC")->fetchAll(PDO::FETCH_ASSOC);

		if (count($result) and in_array($result[0]['type'], $this->levels_to_skip))
		{
			$sem_tree_ids = array_map(create_function('$a', 'return $a["sem_tree_id"];'), $result);
			$result = DBManager::Get()->query("SELECT sem_tree_id, name, type, studip_object_id FROM sem_tree WHERE parent_id IN ('".implode("','", $sem_tree_ids)."') AND type<>0 GROUP BY sem_tree_id ORDER BY name ASC")->fetchAll(PDO::FETCH_ASSOC);
		}

		foreach ($result as &$row)
		{
			if (isset($this->level_filter[ $row['type'] ]))
				$row = $this->level_filter[ $row['type'] ]($row);
			$row['quantity'] = $this->count_assets($row['sem_tree_id']);
		}

		return $result;
	}

	private function count_assets($sem_tree_id)
	{
		$ids = array();
		$nodes = array($sem_tree_id);
		do
		{
			$ids = $nodes;
			$nodes = DBManager::Get()->query("SELECT DISTINCT sem_tree_id FROM sem_tree WHERE parent_id IN ('".implode("','", $nodes)."')")->fetchAll(PDO::FETCH_COLUMN);
		}
		while (count($nodes)>0);

		$query = "SELECT COUNT(DISTINCT sst.seminar_id) FROM seminar_sem_tree sst JOIN seminare s ON s.seminar_id=sst.seminar_id WHERE sem_tree_id IN ('".implode("','", $ids)."')".(!$this->omnipotent?' AND visible=1':'');
		if (!is_null($this->semester_id) and !is_null($this->fachsemester))
			$query = "SELECT COUNT(DISTINCT sst.seminar_id) FROM seminar_sem_tree sst LEFT JOIN seminare s ON sst.seminar_id=s.Seminar_id LEFT JOIN semester_data sd ON sd.semester_id='{$this->semester_id}' JOIN mod_zuordnung mz ON sst.sem_tree_id=mz.sem_tree_id JOIN module m ON mz.modul_abst_id=m.modul_abst_id JOIN mod_fachsem mf ON m.modul_kon_id=mf.modul_kon_id WHERE sst.sem_tree_id IN ('".implode("','", $ids)."') AND s.start_time BETWEEN sd.beginn AND sd.ende AND mf.fachsem_id={$this->fachsemester}".(!$this->omnipotent?' AND visible=1':'');
		elseif (!is_null($this->semester_id))
			$query = "SELECT COUNT(DISTINCT sst.seminar_id) FROM seminar_sem_tree sst LEFT JOIN seminare s ON sst.seminar_id=s.Seminar_id LEFT JOIN semester_data sd ON sd.semester_id='{$this->semester_id}' WHERE sem_tree_id IN ('".implode("','", $ids)."') AND s.start_time BETWEEN sd.beginn AND sd.ende".(!$this->omnipotent?' AND visible=1':'');
		elseif (!is_null($this->fachsemester))
			$query = "SELECT COUNT(DISTINCT sst.seminar_id) FROM seminar_sem_tree sst JOIN mod_zuordnung mz ON sst.sem_tree_id=mz.sem_tree_id JOIN module m ON mz.modul_abst_id=m.modul_abst_id JOIN mod_fachsem mf ON m.modul_kon_id=mf.modul_kon_id JOIN seminare s ON s.seminar_id=sst.seminar_id WHERE sst.sem_tree_id IN ('".implode("','", $ids)."') AND mf.fachsem_id={$this->fachsemester}".(!$this->omnipotent?' AND visible=1':'');

		return DBManager::Get()->query($query)->fetchColumn();
	}

	public function get_assets()
	{
		return array_map(create_function('$a', 'return $a["seminar_id"];'), $this->semester_id === null
			? DBManager::Get()->query("SELECT seminar_id FROM seminar_sem_tree WHERE sem_tree_id='{$this->sem_tree_id}'")->fetchAll(PDO::FETCH_ASSOC)
			: DBManager::Get()->query("SELECT sst.seminar_id FROM seminar_sem_tree sst LEFT JOIN seminare s ON sst.seminar_id=s.Seminar_id LEFT JOIN semester_data sd ON sd.semester_id='{$this->semester_id}' WHERE sem_tree_id='{$this->sem_tree_id}' AND s.start_time BETWEEN sd.beginn AND sd.ende".(!$this->omnipotent?' AND visible=1':''))->fetchAll(PDO::FETCH_ASSOC)
		);
	}

	public function up()
	{
		if ($this->sem_tree_id == 'root')
			return false;

		$data = DBManager::Get()->query("SELECT s0.parent_id, s1.type FROM sem_tree s0 LEFT JOIN sem_tree s1 ON s0.parent_id=s1.sem_tree_id WHERE s0.sem_tree_id='{$this->sem_tree_id}'")->fetch();

		$this->sem_tree_id = $data['parent_id'];
		if (in_array($data['type'], $this->levels_to_skip))
			$this->up();

		return !is_null($this->sem_tree_id);
	}
}
?>